// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool;

import java.text.*;
import java.util.*;
import com.sybase.jaguar.management.*;
import com.sybase.jaguar.util.*;

/**
* Identifier for an entity.  The identifier
* is comprised of a type:name pair.
*/
public class EntityIdentifier
{
    public static final String APP_TYPE      = "Application";   
    public static final String CACHE_TYPE    = "ConnCache";     
    public static final String CLUSTER_TYPE  = "Cluster";       
    public static final String COMP_TYPE     = "Component";     
    public static final String LISTENER_TYPE = "Listener";
    public static final String METHOD_TYPE   = "Method";
    public static final String PACKAGE_TYPE  = "Package";       
    public static final String ROLE_TYPE     = "Role";
    public static final String SERVER_TYPE   = "Server";        
    public static final String SERVLET_TYPE  = "Servlet";       
    public static final String WEBAPP_TYPE   = "WebApplication";

    public static final String[] TYPES =
    {
        APP_TYPE,
        CACHE_TYPE,
        CLUSTER_TYPE,
        COMP_TYPE,
        LISTENER_TYPE,
        METHOD_TYPE,
        PACKAGE_TYPE,
        ROLE_TYPE,
        SERVER_TYPE,
        SERVLET_TYPE,
        WEBAPP_TYPE
    };

    private String _type;
    private String _name;

    public EntityIdentifier(String s) throws EntityIdentifierException
    {
        String type = null;
        String name = null;

        try
        {
            type = s.substring(0, s.indexOf(":"));
        }
        catch (Exception ex)
        {
            throw new EntityIdentifierException();
        }

        setType(type);

        try
        {        
            name = s.substring(s.indexOf(":") + 1);
        }
        catch (Exception ex)
        {
            throw new EntityIdentifierException();
        }

        setName(name);
    }

    public String getType()
    {
        return _type;
    }

    protected void setType(String type) throws EntityIdentifierException
    {
        boolean valid = false;

        for (int i = 0; i < TYPES.length; i++)
        {
            if (type.equalsIgnoreCase(TYPES[i]))
            {
                _type = TYPES[i];
                valid = true;
                break;
            }
        }

        if (!valid)
        {
            throw new EntityIdentifierException();
        }
    }

    public String getName()
    {
        return _name;
    }

    protected void setName(String name) throws EntityIdentifierException
    {
        String s = name.trim();
        
        if (s.length() == 0)
        {
            throw new EntityIdentifierException();
        }

        _name = s;
    }

    public JEntity getEntity(JRepository rep)
    {
        JEntity entity = null;

        if (_type.equals(APP_TYPE))
        {
            entity = rep.getApplication(_name);
        }
        else if (_type.equals(CACHE_TYPE))
        {
            entity = rep.getConnectionCache(_name);
        }
        else if (_type.equals(CLUSTER_TYPE))
        {
            entity = rep.getCluster(_name);
        }
        else if (_type.equals(COMP_TYPE))
        {
            String packName = _name.substring(0, _name.indexOf('/'));
            String compName = _name.substring(_name.indexOf('/') + 1);

            JPackage pack = rep.getPackage(packName);

            if (pack != null)
            {
                entity = pack.getComponent(compName);
            }
        }
        else if (_type.equals(LISTENER_TYPE))
        {
            entity = rep.getListenerProfile(_name);
        }
        else if (_type.equals(METHOD_TYPE))
        {
            String s = _name;

            String packName = s.substring(0, s.indexOf('/'));

            s = s.substring(s.indexOf('/') + 1);

            String compName = s.substring(0, s.indexOf('/'));
            String methodName = s.substring(s.indexOf('/') + 1);

            JPackage pack = rep.getPackage(packName);

            if (pack != null)
            {
                JComponent comp = pack.getComponent(compName);

                if (comp != null)
                {
                    try
                    {
                        entity = comp.getMethod(methodName);
                    }
                    catch (JException jex)
                    {
                        // Probably should handle this, but I don't want
                        // to add it to the signature, since no other
                        // types throw exceptions.  Just asserting for now.
                        Assert.assert(this, false,
                            "Caught a JException while retrieving a JMethod");
                    }
                }
            }
        }
        else if (_type.equals(PACKAGE_TYPE))
        {
            entity = rep.getPackage(_name);
        }
        else if (_type.equals(ROLE_TYPE))
        {
            entity = rep.getRole(_name);
        }
        else if (_type.equals(SERVER_TYPE))
        {
            entity = rep.getServer(_name);
        }
        else if (_type.equals(SERVLET_TYPE))
        {
            entity = rep.getServlet(_name);
        }
        else if (_type.equals(WEBAPP_TYPE))
        {
            entity = rep.getWebApplication(_name);
        }
        else
        {
            Assert.assert(this, false,
                MessageFormat.format("Unexpected entity type: {0}",
                    new String[] { _type }));
        }

        return entity;
    }
}
