// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool;

import java.util.ResourceBundle;
import java.util.Locale;
import java.text.MessageFormat;

/**
 * Resource manager for jagtool.
 */
public class JTResourceManager
{
    private Class defaultClass;

    /**
     * Create a string manager that looks for the ResourceBundle in
     * the package of the defaultClass.
     */
    public JTResourceManager(Class defaultClass)
    {
        this.defaultClass = defaultClass;
    }

    /**
     * Get a localized string.
     */
    public String getString(String key)
    {
        Class stopClass = defaultClass.getSuperclass();
        Class startClass = defaultClass;
        ResourceBundle resources = null;
        boolean globalDone = false;

        for (Class c = startClass; c != stopClass && c != null; 
            c = c.getSuperclass())
        {
            globalDone = (c == defaultClass);

            try
            {
                // Construct the bundle name as LocalStrings in the
                // caller class's package.
                StringBuffer resFileName = new StringBuffer(c.getName().substring(0, 
                    c.getName().lastIndexOf(".")));

                resFileName.append(".ResourceBundle");

                resources = ResourceBundle.getBundle(resFileName.toString());

                if (resources != null)
                {
                    String value = resources.getString(key);

                    if (value != null)
                        return value;
                }
            }
            catch (Exception ex) {}
        }

        // Look for a global resource (defined by defaultClass)
        if (!globalDone)
        {
            return getString(key);
        }
        else
        {
            System.out.println("No local string for " + key);
            return null;
        }
    }

    /**
     * Get a local string for the caller and format the arguments accordingly.
     */
    public String getString(String key, Object arguments[])
    {
        MessageFormat f =
            new MessageFormat(getString(key));

        for (int i = 0; i < arguments.length; i++)
        {
            if (arguments[i] == null)
            {
                arguments[i] = "null";
            }
            else if (!(arguments[i] instanceof String) &&
                     !(arguments[i] instanceof Number) &&
                     !(arguments[i] instanceof java.util.Date))
            {
                arguments[i] = arguments[i].toString();
            }
        }

        return f.format(arguments);
    }
}

