// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool;

import java.util.*;
import com.sybase.jaguar.system.Repository;
import com.sybase.jaguar.system.RepositoryHelper;

public class Utils extends Object
{
    private static JTResourceManager _resources =
    	new JTResourceManager(Utils.class);

    private Utils() {}

    public static Vector arrayToVector(Object[] array)
    {
        Vector v = new Vector();

        for (int i = 0; i < array.length; i++)
        {
            v.addElement(array[i]);
        }

        return v;
    }

    public static String arrayToString(Object[] array)
    {
        return arrayToString(array, " ");
    }

    public static String arrayToString(Object[] array, String delimiter)
    {
        String s = "";

        for (int i = 0; i < array.length; i++)
        {
            s += array[i];

            if (i < (array.length - 1))
                s += delimiter;
        }

        return s;
    }

    public static String formatDescription(String commandName, String desc)
    {
        return _resources.getString("jagtoolCommandDesc", new String[] { commandName, desc });
    }

    public static String formatUsage(String commandName, String usage)
    {
        return _resources.getString("jagtoolCommandUsage", new String[] { commandName, usage });
    }

    public static String formatCommandMsg(String commandName, String msg)
    {
        return _resources.getString("jagtoolCommandMsg", new String[] { commandName, msg });
    }

    public static String formatMissingValueMsg(String optionName)
    {
        return _resources.getString("missingValueMsg", new String[] { optionName });
    }

    public static String formatInvalidValueMsg(String optionName, String invalidValue)
    {
        return _resources.getString("invalidValueMsg", new String[] { optionName, invalidValue });
    }

    public static String formatUnknownOptionMsg(String optionName)
    {
        return _resources.getString("unknownOptionMsg", new String[] { optionName });
    }

    public static String formatInvalidEntityTypeMsg(String invalidType, String[] validTypes)
    {
        return _resources.getString("invalidEntityTypeMsg", new String[] { invalidType, arrayToString(validTypes) });
    }

    public static void checkEntityIsSet(Command command, String propName, EntityIdentifier entity) throws UsageException
    {
        if (entity == null)
        {
            throw new UsageException(Utils.formatMissingValueMsg(propName), command);
        }
    }

    public static void checkEntityType(Command command, EntityIdentifier entity, String[] validTypes) throws UsageException
    {
        boolean valid = false;

        for (int i = 0; i < validTypes.length; i++)
        {
            if (entity.getType().equals(validTypes[i]))
            {
                valid = true;
                break;
            }
        }

        if (valid == false)
        {
            throw new UsageException(formatInvalidEntityTypeMsg(entity.getType(), validTypes), command);
        }
    }

    public static void checkEntityExists(Command command, JTContext context, EntityIdentifier entity) throws JTException
    {
        Repository rep = context.getRepository();

        boolean exists = rep.exists(entity.getType(), entity.getName());

        if (exists == false)
        {
            throw new JTException(_resources.getString("entityNotExistMsg",
                new String[] { entity.getType(), entity.getName() }), command);
        }
    }
}

