// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool.commands;

import java.io.*;
import java.util.*;
import com.sybase.jaguar.management.jagtool.*;
import com.sybase.jaguar.management.*;
import com.sybase.jaguar.importer.*;
import com.sybase.jaguar.packager.*;
import com.sun.enterprise.deployment.*;

public class DeployCommand extends BaseCommand
{
    public static final String TYPE_PROP  = "type";    
    public static final String SKELS_PROP = "skels";   
    public static final String FILE_PROP  = "file";

    public static final String EAR_TYPE      = "ear";
    public static final String WAR_TYPE      = "war";
    public static final String EJBJAR_TYPE   = "ejbjar";
    public static final String JAGJAR_TYPE   = "jagjar";

    private static JTResourceManager _resources =
    	new JTResourceManager(DeployCommand.class);

    private String  _typeProp = EAR_TYPE;
    private boolean _skelsProp = true;
    private String  _fileProp;

    public String getName()
    {
        return _resources.getString("deployName");
    }

    public String getDescription()
    {
        return _resources.getString("deployDesc");
    }

    public String getUsage()
    {
        return _resources.getString("deployUsage");
    }

    public void setType(String typeProp) throws Exception
    {
        if (typeProp.equals(EAR_TYPE) ||
            typeProp.equals(WAR_TYPE) ||
            typeProp.equals(EJBJAR_TYPE) ||
            typeProp.equals(JAGJAR_TYPE))
        {
            _typeProp = typeProp;
        }
        else
        {
            throw new UsageException(Utils.formatInvalidValueMsg(TYPE_PROP, typeProp), this);
        }
    }

    public void setSkels(boolean skelsProp) throws Exception
    {
        _skelsProp = skelsProp;
    }

    public void setFile(String fileProp) throws Exception
    {
        _fileProp = fileProp;
    }

    public void setArgs(String[] args) throws Exception
    {
        for (int i = 0; i < args.length; i++)
        {
            String arg = args[i];

            if (arg.equals("-" + TYPE_PROP))
            {
                try
                {
                    setType(args[i+1]);
                    i++;
                }
                catch (ArrayIndexOutOfBoundsException aioobe)
                {
                    throw new UsageException(this);
                }
            }
            else if (arg.equals("-" + SKELS_PROP))
            {
                try
                {
                    String s = args[i+1];
                    if (s.equalsIgnoreCase("true"))
                    {
                        setSkels(true);
                    }
                    else if (s.equalsIgnoreCase("false"))
                    {
                        setSkels(false);
                    }
                    else
                    {
                        throw new UsageException(Utils.formatInvalidValueMsg(SKELS_PROP, s), this);
                    }
                    i++;
                }
                catch (ArrayIndexOutOfBoundsException aioobe)
                {
                    throw new UsageException(this);
                }
            }
            else if (arg.startsWith("-"))
            {
                throw new UsageException(Utils.formatUnknownOptionMsg(arg), this);
            }
            else
            {
                _fileProp = arg;
                break;
            }
        }
    }

    public void run() throws Exception
    {
        if (_fileProp == null)
        {
            throw new UsageException(Utils.formatMissingValueMsg(FILE_PROP), this);
        }

        JTContext context = getContext();

        JRepository repository =
            new JRepository(context.getUserName(),
                            context.getPassword(),
                            context.getHostName(),
                            context.getPort(),
                            "rw");

        JEarImporter importer = null;

        try
        {
            importer = new JEarImporter(repository, _fileProp);
        }
        catch (Exception ex)
        {
            throw new JTException(_resources.getString("deployExceptionLoadingFile",
                new String[] { _fileProp }), this, ex);
        }

        if (_typeProp.equals(EAR_TYPE))
        {
            deployEar(importer);
        }
        else if (_typeProp.equals(WAR_TYPE))
        {
            deployWar(importer);
        }
        else if (_typeProp.equals(EJBJAR_TYPE))
        {
            deployEJBJar(importer);
        }
        else if (_typeProp.equals(JAGJAR_TYPE))
        {
            deployJagJar(repository, _fileProp);
        }
    }

    protected void deployEar(JEarImporter importer) throws Exception
    {
        DefaultStatusHandler statusHandler =
            new DefaultStatusHandler(getLogger());

        importer.setStatusDialogInterface(statusHandler);
        importer.setGenerateStubsAndSkels(_skelsProp);

        if (!importer.isApplication())
        {
            throw new JTException(_resources.getString("deployInvalidFileTypeMsg", 
                new String[] { TYPE_PROP, _typeProp, _fileProp }), this);
        }

        try
        {
            statusHandler.addConsoleMsg(_resources.getString("deployStartMsg"));

            for (Enumeration f = importer.getEJBBundleDescriptors(); f.hasMoreElements(); )
            {
                EjbBundleDescriptor bdescriptor = 
                    (EjbBundleDescriptor) f.nextElement();

                importer.deployEJBBundle(bdescriptor);
            }

			importer.setEjbReferences();

            for (Enumeration f = importer.getWebBundleDescriptors(); f.hasMoreElements(); )
            {
                WebBundleDescriptor bdescriptor = 
                    (WebBundleDescriptor) f.nextElement();

                importer.deployWebAppBundle(bdescriptor);
            }

            for (Enumeration f = importer.getApplicationClientDescriptors();
                f.hasMoreElements(); )
            {
                ApplicationClientDescriptor descriptor = 
                    (ApplicationClientDescriptor) f.nextElement();

                importer.deployAppClient(descriptor);
            }
        }
        catch (Exception ex)
        {
            statusHandler.addConsoleMsg(_resources.getString("deployFailedMsg"));
            throw ex;
        }
    }

    protected void deployWar(JEarImporter importer) throws Exception
    {
        DefaultStatusHandler statusHandler =
            new DefaultStatusHandler(getLogger());

        importer.setStatusDialogInterface(statusHandler);

        if (!importer.isWar())
        {
            throw new JTException(_resources.getString("deployInvalidFileTypeMsg", 
                new String[] { TYPE_PROP, _typeProp, _fileProp }), this);
        }

        try
        {
            statusHandler.addConsoleMsg(_resources.getString("deployStartMsg"));

            for (Enumeration f = importer.getWebBundleDescriptors(); f.hasMoreElements(); )
            {
                WebBundleDescriptor bdescriptor = 
                    (WebBundleDescriptor) f.nextElement();

                importer.deployWebAppBundle(bdescriptor);
            }
        }
        catch (Exception ex)
        {
            statusHandler.addConsoleMsg(_resources.getString("deployFailedMsg"));
            throw ex;
        }
    }

    protected void deployEJBJar(JEarImporter importer) throws Exception
    {
        DefaultStatusHandler statusHandler =
            new DefaultStatusHandler(getLogger());

        importer.setStatusDialogInterface(statusHandler);
        importer.setGenerateStubsAndSkels(_skelsProp);

        if (!importer.isEJBJar())
        {
            throw new JTException(_resources.getString("deployInvalidFileTypeMsg", 
                new String[] { TYPE_PROP, _typeProp, _fileProp }), this);
        }

        try
        {
            statusHandler.addConsoleMsg(_resources.getString("deployStartMsg"));

            for (Enumeration f = importer.getEJBBundleDescriptors(); f.hasMoreElements(); )
            {
                EjbBundleDescriptor bdescriptor = 
                    (EjbBundleDescriptor) f.nextElement();

                importer.deployEJBBundle(bdescriptor);
            }

			importer.setEjbReferences();
        }
        catch (Exception ex)
        {
            statusHandler.addConsoleMsg(_resources.getString("deployFailedMsg"));
            throw ex;
        }
    }

    protected void deployJagJar(JRepository repository, String fileName) throws Exception
    {
        File file = new File(fileName);

        if (!file.exists())
        {
            throw new JTException(_resources.getString("deployNoSuchFile",
                new String[] { fileName }), this);
        }

        //! tenglund 12/4/00
        //! Always passing type as application for now.  May need
        //! to add a command-line option for this.

        JPackager packager = new JPackager(repository, JFinal.JAGUAR_APPLICATION_TYPE);

        packager.importPackageV2(fileName); 

        if (packager.getErrFlag())
        {
            throw new JTException(_resources.getString("deployJagJarError",
                new String[] { fileName }), this);
        }

        println(_resources.getString("deploySuccessMsg",
            new String[] { fileName }));
    }
}
