// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool.commands;

import java.util.*;
import com.sybase.jaguar.management.jagtool.*;
import com.sybase.jaguar.system.Management;
import com.sybase.jaguar.system.ManagementHelper;
import com.sybase.jaguar.system.Repository;
import com.sybase.jaguar.system.RepositoryHelper;

/**
* Command that determines whether an entity exists.
*/
public class ExistsCommand extends BaseCommand
{
    public static final String ENTITY_PROP = "entity";

    private static JTResourceManager _resources =
    	new JTResourceManager(ExistsCommand.class);

    private EntityIdentifier _entity;

    public String getName()
    {
        return _resources.getString("existsName");
    }

    public String getDescription()
    {
        return _resources.getString("existsDesc");
    }

    public String getUsage()
    {
        return _resources.getString("existsUsage");
    }

    public boolean isHidden()
    {
        return true;
    }

    public void setEntity(String entityProp) throws Exception
    {
        try
        {
            _entity = new EntityIdentifier(entityProp);
        }
        catch (EntityIdentifierException ex)
        {
            throw new UsageException(Utils.formatInvalidValueMsg(ENTITY_PROP, entityProp), this);
        }
    }

    public void setArgs(String[] args) throws Exception
    {
        if (args.length == 0)
        {
            throw new UsageException(this);
        }
        else
        {
            setEntity(args[0]);
        }
    }

    public void run() throws Exception
    {
        JTContext context = getContext();

        boolean exists = true;

        try
        {
            Utils.checkEntityExists(this, context, _entity);
        }
        catch (JTException jtex)
        {
            exists = false;
        }

        if (exists)
        {
            println(_resources.getString("entityExistsMsg",
                new String[] { _entity.getType(), _entity.getName() }));
        }
        else
        {
            println(_resources.getString("entityNotExistMsg",
                new String[] { _entity.getType(), _entity.getName() }));
        }
    }
}
