// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool.commands;

import com.sybase.jaguar.management.common.*;
import com.sybase.jaguar.management.jagtool.*;
import com.sybase.jaguar.management.*;
import com.sybase.jaguar.packager.*;

public class ExportAppCommand extends BaseCommand
{
    private static LocalResourceManagerImpl _resources =
    	new LocalResourceManagerImpl(ExportAppCommand.class);

    private String _appName;
    private String _fileName;

    public String getName()
    {
        return _resources.getString("exportAppName", "export_app");
    }

    public String getHelp()
    {
        return _resources.getString("exportAppHelp", "");
    }

    public String getUsage()
    {
        return _resources.getString("exportAppUsage", "");
    }

    public void run(String[] args) throws Exception
    {
        if (args.length == 0)
        {
            usage();
            return;
        }
        else
        {
            _appName = args[0];
            _fileName = _appName + ".ear";
        }

        JRepository repository =
            new JRepository(JTGlobals.getUserName(),
                            JTGlobals.getPassword(),                            
                            JTGlobals.getHostName(),
                            JTGlobals.getPort(),
                            "rw");

        JApplication app = repository.getApplication(_appName);

        if (app == null)
        {
            System.out.println(_resources.getString("appNotExistMsg",
                "Application {0} does not exist",
                new String[] { _appName }));
            setExitStatus(JTConstants.ERROR_STATUS);
            return;
        }

        JEarPackager packager = null;

        DefaultStatusHandler statusHandler =
            new DefaultStatusHandler();

        try
        {
            packager = new JEarPackager(repository);

            packager.setStatusDialogInterface(statusHandler);

	        packager.exportApplicationEar(_appName, _fileName);
        }
        catch (Exception ex)
        {
            statusHandler.addConsoleMsg(_resources.getString("exportFailedMsg", "Export Failed"));
            throw ex;
        }
    }

}
