// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool.commands;

import com.sybase.jaguar.management.common.*;
import com.sybase.jaguar.management.jagtool.*;
import com.sybase.jaguar.management.*;
import com.sybase.jaguar.packager.*;

public class ExportPackageCommand extends BaseCommand
{
    private static LocalResourceManagerImpl _resources =
    	new LocalResourceManagerImpl(ExportPackageCommand.class);

    private String _packageName;
    private String _fileName;

    public String getName()
    {
        return _resources.getString("exportPackageName", "export_package");
    }

    public String getHelp()
    {
        return _resources.getString("exportPackageHelp", "");
    }

    public String getUsage()
    {
        return _resources.getString("exportPackageUsage", "");
    }

    public void run(String[] args) throws Exception
    {
        if (args.length == 0)
        {
            usage();
            return;
        }
        else
        {
            _packageName = args[0];
            _fileName = _packageName + ".jar";
        }

        JRepository repository =
            new JRepository(JTGlobals.getUserName(),
                            JTGlobals.getPassword(),
                            JTGlobals.getHostName(),
                            JTGlobals.getPort(),
                            "rw");

        JPackage pack = repository.getPackage(_packageName);

        if (pack == null)
        {
            System.out.println(_resources.getString("packageNotExistMsg",
                "Package {0} does not exist",
                new String[] { _packageName }));
            setExitStatus(JTConstants.ERROR_STATUS);
            return;
        }

        JEarPackager packager = null;

        DefaultStatusHandler statusHandler =
            new DefaultStatusHandler();

        try
        {
            packager = new JEarPackager(repository);

            packager.setStatusDialogInterface(statusHandler);

            packager.exportEJBJar(_packageName, _fileName);
        }
        catch (Exception ex)
        {
            statusHandler.addConsoleMsg(_resources.getString("exportFailedMsg", "Export Failed"));
            throw ex;
        }
    }

}
