// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool.commands;

import com.sybase.jaguar.management.common.*;
import com.sybase.jaguar.management.jagtool.*;
import com.sybase.jaguar.management.*;
import com.sybase.jaguar.packager.*;

public class ExportWebAppCommand extends BaseCommand
{
    private static LocalResourceManagerImpl _resources =
    	new LocalResourceManagerImpl(ExportWebAppCommand.class);

    private String _webAppName;
    private String _fileName;

    public String getName()
    {
        return _resources.getString("exportWebAppName", "export_webapp");
    }

    public String getHelp()
    {
        return _resources.getString("exportWebAppHelp", "");
    }

    public String getUsage()
    {
        return _resources.getString("exportWebAppUsage", "");
    }

    public void run(String[] args) throws Exception
    {
        if (args.length == 0)
        {
            usage();
            return;
        }
        else
        {
            _webAppName = args[0];
            _fileName = _webAppName + ".war";
        }

        JRepository repository =
            new JRepository(JTGlobals.getUserName(),
                            JTGlobals.getPassword(),
                            JTGlobals.getHostName(),
                            JTGlobals.getPort(),
                            "rw");

        JWebApplication webApp = repository.getWebApplication(_webAppName);

        if (webApp == null)
        {
            System.out.println(_resources.getString("webAppNotExistMsg",
                "Web Application {0} does not exist",
                new String[] { _webAppName }));
            setExitStatus(JTConstants.ERROR_STATUS);
            return;
        }

        JEarPackager packager = null;

        DefaultStatusHandler statusHandler =
            new DefaultStatusHandler();

        try
        {
            packager = new JEarPackager(repository);

            packager.setStatusDialogInterface(statusHandler);

            packager.exportWar(_webAppName, _fileName);
        }
        catch (Exception ex)
        {
            statusHandler.addConsoleMsg(_resources.getString("exportFailedMsg", "Export Failed"));
            throw ex;
        }
    }

}
