// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool.commands;

import com.sybase.jaguar.management.jagtool.*;
import java.util.*;

public class HelpCommand extends BaseCommand
{
    public static final String COMMAND_PROP = "command";

    private static JTResourceManager _resources =
    	new JTResourceManager(HelpCommand.class);

    private String _commandProp;

    public String getName()
    {
        return _resources.getString("helpName");
    }

    public String getDescription()
    {
        return _resources.getString("helpDesc");
    }

    public String getUsage()
    {
        return _resources.getString("helpUsage");
    }

    public void setCommand(String commandProp)
    {
        _commandProp = commandProp;
    }

    public void setArgs(String[] args) throws Exception
    {
        if (args.length > 0)
        {
            String arg = args[0];

            if (arg.startsWith("-"))
            {
                throw new UsageException(Utils.formatUnknownOptionMsg(arg), this);
            }
            else
            {
                setCommand(arg);
            }
        }
    }

    public void run() throws Exception
    {
        if (_commandProp == null)
        {
            println(Utils.formatUsage(jagtool.instance().getName(), jagtool.instance().getUsage()));
            println(_resources.getString("helpHelpCommand"));
        }
        else
        {
            CommandFactory factory = CommandFactory.instance();

            if (!factory.hasCommand(_commandProp))
            {
                throw new UsageException(_resources.getString("invalidCommandMsg",
                    new String[] { _commandProp }), this);
            }
            else
            {
                Command command = factory.createCommand(_commandProp);

                println();
                println(Utils.formatDescription(command.getName(), command.getDescription()));
                println();
                println(Utils.formatUsage(command.getName(), command.getUsage()));
            }
        }
    }
}
