// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool.commands;

import java.util.*;
import com.sybase.jaguar.management.jagtool.*;
import com.sybase.jaguar.system.Management;
import com.sybase.jaguar.system.ManagementHelper;
import com.sybase.jaguar.system.Repository;
import com.sybase.jaguar.system.RepositoryHelper;

/**
* Command that lists entities in a Jaguar server.
*/
public class ListCommand extends BaseCommand
{
    public static final String TYPE_PROP   = "type";  
    public static final String ENTITY_PROP = "entity";

    private static JTResourceManager _resources =
    	new JTResourceManager(ListCommand.class);

    private String _typeProp;
    private EntityIdentifier _entity;

    public String getName()
    {
        return _resources.getString("listName");
    }

    public String getDescription()
    {
        return _resources.getString("listDesc");
    }

    public String getUsage()
    {
        return _resources.getString("listUsage");
    }

    public void setType(String typeProp) throws Exception
    {
        _typeProp = typeProp;
    }

    public void setEntity(String entityProp) throws Exception
    {
        try
        {
            _entity = new EntityIdentifier(entityProp);
        }
        catch (EntityIdentifierException ex)
        {
            throw new UsageException(Utils.formatInvalidValueMsg(ENTITY_PROP, entityProp), this);
        }
    }

    public void setArgs(String[] args) throws Exception
    {
        if (args.length == 0)
        {
            throw new UsageException(this);
        }
        else if (args.length == 1)
        {
            if (args[0].indexOf(":") != -1)
            {
                setEntity(args[0]);
            }
            else
            {
                setType(args[0]);
            }
        }
        else if (args.length == 2)
        {
            setType(args[0]);
            setEntity(args[1]);
        }
        else
        {
            throw new UsageException(this);
        }
    }

    public void run() throws Exception
    {
        JTContext context = getContext();
        Repository rep = context.getRepository();

        if (_entity != null)
        {
            Utils.checkEntityExists(this, context, _entity);
        }

        if (_typeProp == null && _entity == null)
        {
            throw new UsageException(this);
        }
        else if (_typeProp != null && _entity == null)
        {
            String[][] items = rep.items(_typeProp, null);

            for (int i = 0; i < items.length; i++)
            {
                println(items[i][0] + " " + items[i][2] + " " + items[i][3]);
            }
        }
        else if (_typeProp == null && _entity != null)
        {            
            String[][] items = rep.items(_entity.getType(), _entity.getName());

            for (int i = 0; i < items.length; i++)
            {
                println(items[i][0] + " " + items[i][2] + " " + items[i][3]);
            }
        }
        else if (_typeProp != null && _entity != null)
        {            
            String[][] items = rep.items(_entity.getType(), _entity.getName());

            for (int i = 0; i < items.length; i++)
            {
                if (items[i][0].equalsIgnoreCase(_typeProp))
                {
                    println(items[i][0] + " " + items[i][2] + " " + items[i][3]);
                }
            }
        }
    }
}
