// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool.commands;

import java.util.*;
import com.sybase.jaguar.management.jagtool.*;
import com.sybase.jaguar.system.Property;
import com.sybase.jaguar.system.PropertyHelper;
import com.sybase.jaguar.system.Repository;
import com.sybase.jaguar.system.RepositoryHelper;

/**
* Command that displays properites of an entity.
*/
public class PropsCommand extends BaseCommand
{
    public static final String ENTITY_PROP = "entity";

    public static final String[] VALID_ENTITY_TYPES = EntityIdentifier.TYPES;

    private static JTResourceManager _resources =
    	new JTResourceManager(PropsCommand.class);

    private EntityIdentifier _entity;

    public String getName()
    {
        return _resources.getString("propsName");
    }

    public String getDescription()
    {
        return _resources.getString("propsDesc");
    }

    public String getUsage()
    {
        return _resources.getString("propsUsage");
    }

    public void setEntity(String entityProp) throws Exception
    {
        try
        {
            _entity = new EntityIdentifier(entityProp);
        }
        catch (EntityIdentifierException ex)
        {
            throw new UsageException(Utils.formatInvalidValueMsg(ENTITY_PROP, entityProp), this);
        }
    }

    public void setArgs(String[] args) throws Exception
    {
        if (args.length == 0)
        {
            throw new UsageException(this);
        }

        if (args.length > 0)
        {
            setEntity(args[0]);
        }
    }

    public void run() throws Exception
    {
        JTContext context = getContext();

        Utils.checkEntityIsSet(this, ENTITY_PROP, _entity);
        Utils.checkEntityType(this, _entity, VALID_ENTITY_TYPES);
        Utils.checkEntityExists(this, context, _entity);

        Repository rep = context.getRepository();
        Property[] props = rep.lookup(_entity.getType(), _entity.getName());

        for (int i = 0; props != null && i < props.length; i++)
        {
            println(props[i].name + " " + props[i].value);
        }
    }
}
