// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool.commands;

import java.util.*;
import com.sybase.jaguar.management.jagtool.*;
import com.sybase.jaguar.system.Management;
import com.sybase.jaguar.system.ManagementHelper;
import com.sybase.jaguar.system.Repository;
import com.sybase.jaguar.system.RepositoryHelper;

/**
* Command that refreshes entities in a Jaguar server.
*/
public class RefreshCommand extends BaseCommand
{
    public static final String ENTITY_PROP = "entity";

    public static final String[] VALID_ENTITY_TYPES =
    {
        EntityIdentifier.APP_TYPE,
        EntityIdentifier.COMP_TYPE,
        EntityIdentifier.PACKAGE_TYPE,
        EntityIdentifier.SERVER_TYPE,
        EntityIdentifier.WEBAPP_TYPE
    };

    private static JTResourceManager _resources =
    	new JTResourceManager(RefreshCommand.class);

    private EntityIdentifier _entity;

    public String getName()
    {
        return _resources.getString("refreshName");
    }

    public String getDescription()
    {
        return _resources.getString("refreshDesc");
    }

    public String getUsage()
    {
        return _resources.getString("refreshUsage");
    }

    public void setEntity(String entityProp) throws Exception
    {
        try
        {
            _entity = new EntityIdentifier(entityProp);
        }
        catch (EntityIdentifierException ex)
        {
            throw new UsageException(Utils.formatInvalidValueMsg(ENTITY_PROP, entityProp), this);
        }
    }

    public void setArgs(String[] args) throws Exception
    {
        if (args.length == 0)
        {
            throw new UsageException(this);
        }
        else
        {
            String arg = args[0];

            if (arg.startsWith("-"))
            {
                throw new UsageException(Utils.formatUnknownOptionMsg(arg), this);
            }
            else
            {
                setEntity(arg);
            }
        }
    }
    
    public void run() throws Exception
    {
        JTContext context = getContext();

        Utils.checkEntityIsSet(this, ENTITY_PROP, _entity);
        Utils.checkEntityType(this, _entity, VALID_ENTITY_TYPES);
        Utils.checkEntityExists(this, context, _entity);

        Management management = context.getManagement();
        management.refresh(_entity.getType(), _entity.getName());
        println(_resources.getString("refreshedEntityMsg",
            new String[] { _entity.getType(), _entity.getName() }));
    }
}
