// 
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 2000.
// All rights reserved
// 
package com.sybase.jaguar.management.jagtool;

import java.io.*;
import java.util.*;

public class jagtool extends BaseCommand
{
    public static final String ARGS_PROP = "args";

    private static JTResourceManager _resources =
    	new JTResourceManager(jagtool.class);

    private static jagtool _instance;

    private String[] _args;

    public static void main(String[] args)
    {
        int exitStatus = JTConstants.SUCCESS_STATUS;

        _instance = new jagtool();

        try
        {
            _instance.setArgs(args);
            _instance.run();
        }
        catch (UsageException uex)
        {
            exitStatus = JTConstants.ERROR_STATUS;

            Command command = uex.getCommand();

            if (uex.getMessage() != null)
            {
                System.out.println(Utils.formatCommandMsg(command.getName(), uex.getMessage()));
                System.out.println();
            }

            System.out.println(Utils.formatUsage(command.getName(), command.getUsage()));
        }
        catch (JTException jtex)
        {
            exitStatus = JTConstants.ERROR_STATUS;

            Command command = jtex.getCommand();
            String msg = jtex.getMessage();

            if (command != null && msg != null)
            {
                System.out.println(Utils.formatCommandMsg(command.getName(), msg));
            }
            else if (msg != null)
            {
                System.out.println(msg);
            }

            Exception ex = jtex.getNestedException();

            if (ex != null)
            {
                ex.printStackTrace();
            }
        }
        catch (Exception ex)
        {
            exitStatus = JTConstants.ERROR_STATUS;
            ex.printStackTrace();
        }

        System.exit(exitStatus);
    }

    public static jagtool instance()
    {
        return _instance;
    }

    public jagtool()
    {
        JTContext context = new JTContext();
        setContext(context);
        DefaultLogger logger = new DefaultLogger();
        setLogger(logger);
    }

    public String getName()
    {
        return _resources.getString("jagtoolName");
    }

    public String getDescription()
    {
        return _resources.getString("jagtoolDesc");
    }

    public String getUsage()
    {
        String usage = _resources.getString("jagtoolUsage");

        CommandFactory factory = CommandFactory.instance();

        Vector names = factory.getCommandNames();

        Collections.sort(names);

        for (int i = 0; i < names.size(); i++)
        {
            Command command = factory.createCommand((String)names.elementAt(i));

            if (!command.isHidden())
            {
                usage += Utils.formatDescription(command.getName(), command.getDescription());
                usage += System.getProperty("line.separator");
            }
        }

        return usage;
    }

    public boolean isHidden()
    {
        return true;
    }

    public void setArgs(String[] args) throws Exception
    {
        _args = args;
    }

    public void run() throws Exception
    {
        registerCommands();

        Vector argv = Utils.arrayToVector(_args);

        if (argv.size() == 0)
        {
            throw new UsageException(this);
        }

        boolean done = false;

        while (!done && argv.size() > 0)
        {
            String arg = (String)argv.elementAt(0);

            if (arg.equals(JTConstants.HOST_FLAG) ||
                arg.equals(JTConstants.HOST_FLAG2))
            {
                if (argv.size() > 1)
                {
                    getContext().setHostName((String)argv.elementAt(1));
                    argv.removeElementAt(0);
                    argv.removeElementAt(0);
                }
                else
                {
                    throw new UsageException(this);
                }
            }
            else if (arg.equals(JTConstants.PORT_FLAG) ||
                     arg.equals(JTConstants.PORT_FLAG2))
            {
                if (argv.size() > 1)
                {
                    getContext().setPort((String)argv.elementAt(1));
                    argv.removeElementAt(0);
                    argv.removeElementAt(0);
                }
                else
                {
                    throw new UsageException(this);
                }
            }
            else if (arg.equals(JTConstants.USER_FLAG) ||
                     arg.equals(JTConstants.USER_FLAG2))
            {
                if (argv.size() > 1)
                {
                    getContext().setUserName((String)argv.elementAt(1));
                    argv.removeElementAt(0);
                    argv.removeElementAt(0);
                }
                else
                {
                    throw new UsageException(this);
                }
            }
            else if (arg.equals(JTConstants.PASSWORD_FLAG) ||
                     arg.equals(JTConstants.PASSWORD_FLAG2))
            {
                if (argv.size() > 1)
                {
                    getContext().setPassword((String)argv.elementAt(1));
                    argv.removeElementAt(0);
                    argv.removeElementAt(0);
                }
                else
                {
                    throw new UsageException(this);
                }
            }
            else
            {
                done = true;
            }
        }

        if (argv.size() == 0)
        {
            throw new UsageException(this);
        }
        else
        {
            String name = (String)argv.elementAt(0);

            argv.removeElementAt(0);

            CommandFactory factory = CommandFactory.instance();

            if (!factory.hasCommand(name))
            {
                throw new UsageException(this);
            }
            else
            {                    
                Command command = factory.createCommand(name);

                // Special handling for BaseCommand

                if (command instanceof BaseCommand)
                {
                    BaseCommand bc = (BaseCommand)command;
                    bc.setContext(getContext());
                    bc.setLogger(getLogger());
                }

                String[] commandArgs = new String[argv.size()];
                argv.copyInto(commandArgs);
                command.setArgs(commandArgs);
                command.run();
            }
        }
    }

    public void registerCommands() throws Exception
    {
        InputStream istr = getClass().getClassLoader().getSystemResourceAsStream(getContext().getCommandFile());

        if (istr == null)
        {
            throw new JTException(_resources.getString("cannotFindCommandsFileMsg",
                new String[] { getContext().getCommandFile() }), this);
        }        
        else
        {
            Properties commands = new Properties();

            try
            {
                commands.load(istr);
            }
            catch (IOException ioex)
            {
                throw new JTException(_resources.getString("cannotLoadCommandsFileMsg",
                    new String[] { getContext().getCommandFile() }), this, ioex);
            }

            for (Enumeration names = commands.propertyNames(); names.hasMoreElements(); )
            {
                String commandName = (String)names.nextElement();
                String className = commands.getProperty(commandName);

                if (className == null)
                {
                    System.out.println(Utils.formatCommandMsg(this.getName(), 
                        _resources.getString("noClassSpecifiedForCommandMsg",
                        new Object[] { commandName })));
                }
                else
                {
                    Class clas = null;

                    try
                    {
                        clas = Class.forName(className);
                    }
                    catch (ClassNotFoundException cnfex)
                    {
                        System.out.println(Utils.formatCommandMsg(this.getName(),
                            _resources.getString("cannotLoadCommandClassMsg",
                            new Object[] { className, commandName })));
                    }

                    if (clas != null)
                    {
                        try
                        {
                            CommandFactory factory = CommandFactory.instance();
                            factory.registerCommand(clas);                        
                        }
                        catch (Exception ex)
                        {
                            System.out.println(Utils.formatCommandMsg(this.getName(),
                                _resources.getString("cannotRegisterCommandClassMsg",
                                new Object[] { className, commandName })));
                        }
                    }
                }
            }
        }
    }
}

