package com.sybase.jaguar.sample.components;


/**
* Imports
*/

import java.io.*;
import java.util.*;
import java.sql.*;

import com.sybase.jaguar.util.*;
import com.sybase.jaguar.sql.*;
import com.sybase.jaguar.server.*;
import com.sybase.jaguar.jcm.*;
import com.sybase.jaguar.beans.enterprise.*;

/**
* This class is a superclass for all Jaguar components.  This class
* has methods to handle creating and maintaining connections and connection
* caches.  As well, this allows you to run your component in a DEBUG mode.
*
* 
* @author David Lance Wolf
* @author Gerry Moses, Sybase Professional Services
* @version 1.0
*
* copyright 1998 Sybase Inc, ALL RIGHTS RESERVED
* 
*/

public class JaguarBeanFramework extends JaguarComponentFramework implements ServerBean
{
	private InstanceContext state;
	private boolean isCommitable = true;

	/**
	* Activates the component and builds a connection. In an instance pooled, early deactivated
	* component, activate() is called before each method invocation as the already instantiated
	* component enters the Activate state.
	*
	* @param InstanceContext state
	* @param String notUsed
	*
	* @throws EnterpriseBeanException
	*
	*/

	public JaguarBeanFramework() throws Exception
	{
		super();
	}

	public JaguarBeanFramework(String user, String pass, String url) throws Exception
	{
		super(user, pass, url);
	}

	public JaguarBeanFramework(File file) throws Exception
	{
		super(file);
	}

	public JaguarBeanFramework(String name) throws Exception
	{
		super(name);
	}

	public void activate(InstanceContext ctx, String notUsed) throws EnterpriseBeanException
	{
		state = ctx;
		if(connection == null)
		{
			try
			{
				setConnection(WAIT_FOR_AVAILABLE_CONNECTION);
				return;
			}
			catch(Exception e)
			{
				e.printStackTrace();
				throw new EnterpriseBeanException(e.getMessage());
			}
		}
	}


	protected void completeWork()
	{
		isCommitable = true;
		state.completeWork();
	}

	protected void rollbackWork()
	{
		state.rollbackWork();
	}

	protected void disallowCommit()
	{
		isCommitable = false;
		state.continueWork();
	}

	protected void continueWork()
	{
		isCommitable = true;
		state.continueWork();
	}

	/**
	* Informs the Jaguar Server that this component does not wishe to be instance
	* pooled.  
	*
	* @returns boolean Indicating if this component can be reused
	*
	*/

	public boolean canReuse()
	{
		return false;
	}

	/**
	* Deactivates the component. Any Exceptions thrown in deactivate() will cause the 
	* transaction to roll back.
	*
	* @throws EnterpriseBeanException
	*
	*/

	public void deactivate() throws EnterpriseBeanException
	{
		if(!isCommitable)
		{
			throw new EnterpriseBeanException("Primitve disallowCommit() called last.  Transaction being rolled back");
		}
		try
		{
			dropConnection();
			return;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new EnterpriseBeanException(e.getMessage());
		}
	}


	/**
	* Destroys the component.  This is a good place to clean up memory.
	*
	*/

	public void destroy()
	{
		destroyCacheContext();
	}
}
