package com.sybase.jaguar.sample.events;

import org.omg.CORBA.*;
import SessionManager.*;
import java.lang.Object;

public class StockClient implements StockEventListener
{
    public static void main(String[] args) throws Exception
    {
        checkUsage(args);
        new StockClient().run(args[0], Integer.parseInt(args[1]), args[2], args[3],
                              args[4], args);
    }

    static void checkUsage(String[] args)
    {
        if (args.length < 5)
        {
            System.out.println("Usage: java EventSamples.StockClient <host> <port> <user> <password> <command>");
            System.out.println(" where <command> is one of:");
            System.out.println("       add <stock> <minValue> <maxValue>");
            System.out.println("       remove <stock> <minValue> <maxValue>");
            System.out.println("       list");
            System.out.println("       run");
            System.exit(0);
        }
    }
    
    StockFilter makeFilter(String[] args)
    {
        String stock = args[5];
        double minValue = Long.parseLong(args[6]);
        double maxValue = Long.parseLong(args[7]);
        return new StockFilter(stock, minValue, maxValue);
    }

    void run(String host, int port, String user, String password, String command, String[] options) throws Exception
    {
        String[] args = { "-org.omg.CORBA.ORBClass", "com.sybase.CORBA.ORB" };
        ORB orb = ORB.init(args, null);
        Manager manager = ManagerHelper.narrow(orb.string_to_object("iiop://" + host + ":" + port));
        Session session = manager.createSession(user, password);
        StockManager stockManager = StockManagerHelper.narrow(session.create("EventSamples/StockManager"));
        
        if (command.equals("add"))
        {
            stockManager.addStockFilter(makeFilter(options));
        }
        else if (command.equals("remove"))
        {
            stockManager.addStockFilter(makeFilter(options));
        }
        else if (command.equals("list"))
        {
            StockFilter[] filters = stockManager.listStockFilters();
            System.out.println("Current stock filters for user " + user + ":");
            for (int i = 0; i < filters.length; i++)
            {
                System.out.println(filters[i].stock
                                   + "\t" + filters[i].minValue
                                   + "\t" + filters[i].maxValue);
            }
        }
        else
        {
            new PullStockEvent(this, stockManager, 0).start();
            for (;;)
            {
                System.out.println("Simulated user interaction...");
                Thread.sleep(10000);
            }
        }
    }

    public void stockEvent(StockEvent event)
    {
        System.err.println("Notification: stock = " + event.stock
                           + ", value = " + event.value);
    }
}
