package com.sybase.jaguar.sample.events;

import java.util.Hashtable;
import java.util.Enumeration;

public class StockManagerImpl
{
    private static int _counter = 0;
    
    private static Object _lock = new Integer(0);
    
    private static Hashtable _filters = new Hashtable(); // keyed by username

    private static Hashqueue _events = new Hashqueue(1000);
    
    private String user()
    {
        try
        {
            return com.sybase.jaguar.server.Jaguar.getUserName();
        }
        catch (com.sybase.jaguar.util.JException je)
        {
            je.printStackTrace();
            throw new org.omg.CORBA.NO_PERMISSION();
        }
    }
    
    private String key(StockFilter filter)
    {
        return filter.stock + "<" + filter.minValue + ">" + filter.maxValue;
    }
    
    public int newStockEventId()
    {
        synchronized(_lock)
        {
            int id = ++_counter;
            return id;
        }
    }

    public void addStockFilter
        (StockFilter filter)
    {
        String key = key(filter);
        synchronized(_filters)
        {
            Hashtable userFilters = (Hashtable)_filters.get(user());
            if (userFilters == null)
            {
                userFilters = new Hashtable();
                _filters.put(user(), userFilters);
            }
            userFilters.put(key, filter);
        }
    }

    public void removeStockFilter
        (StockFilter filter)
    {
        String key = key(filter);
        synchronized(_filters)
        {
            Hashtable userFilters = (Hashtable)_filters.get(user());
            if (userFilters != null)
            {
                userFilters.remove(key);
            }
        }
    }

    public StockFilter[] listStockFilters()
    {
        Hashtable userFilters;
        StockFilter[] listFilters;
        synchronized(_filters)
        {
            userFilters = (Hashtable)_filters.get(user());
            if (userFilters == null)
            {
                userFilters = new Hashtable();
            }
        }
        synchronized(userFilters)
        {
            int n = userFilters.size();
            listFilters = new StockFilter[userFilters.size()];
            Enumeration e = userFilters.elements();
            for (int i = 0; i < n; i++)
            {
                StockFilter filter = (StockFilter)e.nextElement();
                listFilters[i] = filter;
            }
        }
        return listFilters;
    }

    public void pushStockEvent
        (StockEvent event)
    {
        synchronized(_events)
        {
            _events.put(new Long(event.id), event);
            _events.notifyAll();
        }
    }

    public StockEvent pullStockEvent
        (int lastId)
    {
        StockFilter[] listFilters = listStockFilters();
        for (;;)
        {
            try
            {
                synchronized(_events)
                {
                    int oldest = _events.oldest().intValue();
                    int newest = _events.newest().intValue();
                    if (oldest > lastId)
                    {
                        lastId = oldest;
                    }
                    for (long id = lastId; id <= newest; id++)
                    {
                        StockEvent event = (StockEvent)_events.get(new Long(id));
                        if (event != null)
                        {
                            for (int i = 0; i < listFilters.length; i++)
                            {
                                StockFilter filter = listFilters[i];
                                if (event.stock.equals(filter.stock))
                                {
                                    if (event.value < filter.minValue || event.value > filter.maxValue)
                                    {
                                        return event;
                                    }
                                }
                            }
                        }
                    }
                    lastId = newest + 1;
                    _events.wait();
                }
            }
            catch (InterruptedException ignore)
            {
            }
        }
    }
}
