-- Assumes already in migration repository database
/*    Base Tables  */
if (exists (select 1 from sysobjects where name = 'mt_id'  
	and type = 'U')) 
	drop table mt_id 
go 
create table mt_id 
( 
	next_id 	numeric(10,0) identity
) 
go 

grant all on mt_id to public
go

if (exists (select 1 from sysobjects where name = 'mt_repositorydevices'  
	and type = 'U')) 
	drop table mt_repositorydevices 
go 
create table mt_repositorydevices 
( 
	session_id 	numeric(10,0), 
	device	varchar(35), 
	size_mb   int, 
	used	int default 0 
		/*
		** indicates amount of space allocated out of 
		** size_mb to repository 
		*/
) 
go 
	
grant all on mt_repositorydevices to public
go

if (exists (select 1 from sysobjects where name = 'mt_sessions'  
	and type = 'U')) 
	drop table mt_sessions 
go 
create table mt_sessions 
( 
	session_id 	numeric(10,0) identity constraint unique_id unique, 
	src_server	varchar(100), 
	tgt_server	varchar(100), 
	migrate_server_info	tinyint default 0  
		constraint valid_choice check (migrate_server_info in (0,1,2)), 
	status		int default 0 
		constraint valid_session_status check (status < 4), 
		-- Status of migrating server info 
	selected	tinyint default 0 
		constraint session_yes_no check (selected in (0,1)), 
		-- Has this session been selected for setup ?
	constraint pk primary key(src_server, tgt_server) 
) 
	
go 
	
grant all on mt_sessions to public
go

if (exists (select 1 from sysobjects where name = 'mt_paths'  
	and type = 'U')) 
	drop table mt_paths 
go 
create table mt_paths 
( 
	path_id 	numeric(10,0) identity constraint unique_id unique, 
	session_id 	numeric(10,0), 
	src_db		varchar(35), 
	tgt_db		varchar(35), 
	max_copy_threads  int default 5, 
	max_idx_threads   int default 1, 
	pdb_name	varchar(35) null, 
	pdb_device	varchar(35) null, 
	estsize_mb   int default 0, 
	estrepsize_mb   int default 0, 
	size_mb   int, 
	allocdev	tinyint default 0 
		constraint allocdev_yes_no check (allocdev in (0,1)), 
			-- indicates if device  has been allocated for this path 
	status		int default 0 
		constraint valid_path_status check (status < 4),  
			-- Status of migration db info 
	allocrep	tinyint default 0 
		constraint allocrep_yes_no check (allocrep in (0,1)), 
			-- indicates if repository space has been allocated for this 
	constraint pk primary key(session_id, src_db, tgt_db) 
) 
	
go 
	
grant all on mt_paths to public
go

if (exists (select 1 from sysobjects where name = 'mt_dbs'  
	and type = 'U')) 
	drop table mt_dbs 
go 
create table mt_dbs 
( 
	server	varchar(100), 
	db		varchar(35), 
	selected	tinyint default 0 
	constraint db_busy_yes_no check (selected in (0,1)), 
		-- indicates if db is selected for migration 
) 
	
go 
	
grant all on mt_dbs to public
go

create unique clustered index cidx on mt_dbs(server,db) with ignore_dup_key 
go 
	
if (exists (select 1 from sysobjects where name = 'mt_tasks'  
	and type = 'U')) 
	drop table mt_tasks 
go 
create table mt_tasks 
( 
	task_id 		numeric(10,0) identity constraint unique_id unique, 
	path_id 		numeric(10,0) null, 
	session_id 		numeric(10,0) null, 
	type			char(3), 
			/*
			** The next 3 columns store the owner, table_name and name
			** in the quoted format i.e. parsed with enclosing double quotes.
			** This is how the index, UDD, user/login name is stored in catalog.
			** For others this will be the catalog value enclosed in double 
			** quotes for quoted identifiers. 
			*/
	owner			varchar(50) null, 
	table_name		varchar(50) null, 
	name			varchar(50), 

	status			int default 0 
		constraint valid_task_status check (status < 4),  
	selected		tinyint default 1 
		constraint task_selected_yes_no check (selected in (0,1)), 
			/*
			** indicates if task  has been selected for migration or 
			** validation, SInce this will be inserted when this is selected
			** this defaults to 0. When a path is successfully selected for 
			** migration all tasks for that path must be set to 0 as 
			** concurrent migration is not allowed for the same path
			*/
	done	    	tinyint default 0 
		constraint task_done_yes_no check (done in (0,1)), 
			-- indicates if task  has been processed in current run.
	constraint unique_key unique clustered
		(session_id, path_id, type, owner, table_name, name)
) 
	
go 
	
grant all on mt_tasks to public
go

/*
** All the referential constraints
** Disabled as these are not really useful
** and this will avoid dependencies during recreation of repository
** if repository creation fails in the middle
*/

--alter table mt_paths add
--		constraint path_to_session_ref references mt_sessions(session_id)
--go
--
--alter table mt_tasks add
--		constraint task_to_path_ref references mt_paths(path_id), 
--go
--
--alter table mt_tasks add
--		constraint task_to_session_ref references mt_sessions(session_id), 
--go

--   Useful Views 
if (exists (select 1 from sysobjects where name = 'v_selected_items'  
	and type = 'V')) 
	drop view v_selected_items 
go 
create view v_selected_items as
	select  session_id, path_id, type, owner, table_name, name, status 
	from mt_tasks 
	where selected = 1 
	
	
go 

grant select on v_selected_items to public
go

if (exists (select 1 from sysobjects where name = 'v_failed_items'  
	and type = 'V')) 
	drop view v_failed_items 
go 
create view v_failed_items as
	select  session_id, path_id, type, owner, table_name, name 
	from mt_tasks 
	where selected = 1 
	and 	status < 0 
	
	
go 
	
grant select on v_failed_items to public
go

if (exists (select 1 from sysobjects where name = 'v_successful_items'  
	and type = 'V')) 
	drop view v_successful_items 
go 
create view v_successful_items as
	select  session_id, path_id, type, owner, table_name, name 
	from mt_tasks 
	where selected = 1 
	and 	status = 2 
	
go 
	
grant select on v_successful_items to public
go

if (exists (select 1 from sysobjects where name = 'v_validated_items'  
	and type = 'V')) 
	drop view v_validated_items 
go 
create view v_validated_items as
	select  session_id, path_id, type, owner, table_name, name 
	from mt_tasks 
	where selected = 1 
	and 	status = 3 
	
go 
	
grant select on v_validated_items to public
go

if (exists (select 1 from sysobjects where name = 'v_items_in_progress'  
	and type = 'V')) 
	drop view v_items_in_progress 
go 
create view v_items_in_progress as
	select  session_id, path_id, type, owner, table_name, name 
	from mt_tasks 
	where selected = 1 
	and 	status = 1 
	
go 
grant select on v_items_in_progress to public
go

if (exists (select 1 from sysobjects where name = 'v_items_not_started'  
	and type = 'V')) 
	drop view v_items_not_started 
go 
create view v_items_not_started as
	select  session_id, path_id, type, owner, table_name, name 
	from mt_tasks 
	where selected = 1 
	and 	status = 0 
	
go 
grant select on v_items_not_started to public
go

--   Useful Stored Procs 



--   finally create the control table
if (exists (select 1 from sysobjects where name = 'mt_control'
    and type = 'U'))
    drop table mt_control
go
create table mt_control
(
    code    varchar(255),
    intVal  int default 0,
    strVal  varchar(1024) default "",
    constraint pk primary key (code)
) lock datarows
go

grant all on mt_control to public
go

