/*
 * $Id: EJBHome.java,v 1.7 1999/10/30 08:08:19 tcfujii Exp $
 *
 * Copyright 1998, 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package javax.ejb;

import java.rmi.*;

/**
 * The EJBHome interface is extended by all enterprise
 * Bean's home interfaces. An enterprise Bean's home interface defines the
 * methods that allow a client to create, find, and remove EJB objects.
 *
 * <p> Each enterprise Bean has a home interface. The home interface must
 * extend the javax.ejb.EJBHome interface, and define the enterprise Bean
 * type specific create and finder methods (session Beans do not have finders).
 *
 * <p> The home interface is defined by the enterprise Bean provider and 
 * implemented by the enterprise Bean container.
 */
public interface EJBHome extends Remote {

    /**
     * Remove an EJB object identified by its handle.
     *
     * @exception RemoveException Thrown if the enterprise Bean or
     *    the container does not allow the client to remove the object.
     *
     * @exception RemoteException Thrown when the method failed due to a
     *    system-level failure.
     */
    void remove(Handle handle) throws RemoteException, RemoveException;

    /**
     * Remove an EJB object identified by its primary key.
     *
     * <p>This method can be used only for an entity bean. An attempt
     * to call this method on a session bean will result in RemoteException.
     *
     * @exception RemoveException Thrown if the enterprise Bean or
     *    the container does not allow the client to remove the object.
     *
     * @exception RemoteException Thrown when the method failed due to a
     *    system-level failure.
     */
    void remove(Object primaryKey) throws RemoteException, RemoveException;

    /**
     * Obtain the EJBMetaData interface for the enterprise Bean. The
     * EJBMetaData interface allows the client to obtain information about
     * the enterprise Bean.
     *
     * <p> The information obtainable via the EJBMetaData interface is
     * intended to be used by tools.
     *
     * @return The enterprise Bean's EJBMetaData interface.
     *
     * @exception RemoteException Thrown when the method failed due to a
     *    system-level failure.
     */
    EJBMetaData getEJBMetaData() throws RemoteException;

    /**
     * Obtain a handle for the home object. The handle can be used at 
     * later time to re-obtain a reference to the home object, possibly 
     * in a different Java Virtual Machine.
     *
     * @return A handle for the home object.
     *
     * @exception RemoteException Thrown when the method failed due to a
     *    system-level failure.     
     */
    HomeHandle getHomeHandle() throws RemoteException;
}
