/*
 * $Id: EntityContext.java,v 1.6 1999/10/30 08:08:24 tcfujii Exp $
 *
 * Copyright 1998, 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package javax.ejb;

import java.util.*;
import java.security.Identity;

/**
 * The EntityContext interface provides an instance with access to the 
 * container-provided runtime context of an entity enterprise Bean instance. 
 * The container passes the EntityContext interface to an entity enterprise 
 * Bean instance after the instance has been created. 
 *
 * <p> The EntityContext interface remains associated with the instance for 
 * the lifetime of the instance. Note that the information that the instance
 * obtains using the EntityContext interface (such as the result of the
 * getPrimaryKey() method) may change, as the container assigns the instance
 * to different EJB objects during the instance's life cycle.
 */
public interface EntityContext extends EJBContext
{
    /**
     * Obtain a reference to the EJB object that is currently associated with 
     * the instance.
     *
     * <p> An instance of an entity enterprise Bean can call this method only
     * when the instance is associated with an EJB object identity, i.e.
     * in the ejbActivate, ejbPassivate, ejbPostCreate method, ejbRemove,
     * ejbLoad, ejbStore, and business methods.
     *
     * <p> An instance can use this method, for example, when it wants to
     * pass a reference to itself in a method argument or result.
     *
     * @return The EJB object currently associated with the instance.
     *
     * @exception IllegalStateException Thrown if the instance invokes this
     *    method while the instance is in a state that does not allow the
     *    instance to invoke this method.
     */
    EJBObject getEJBObject() throws IllegalStateException;

    /**
     * Obtain the primary key of the EJB object that is currently
     * associated with this instance.
     *
     * <p> An instance of an entity enterprise Bean can call this method only
     * when the instance is associated with an EJB object identity, i.e.
     * in the ejbActivate, ejbPassivate, ejbPostCreate method, ejbRemove,
     * ejbLoad, ejbStore, and business methods.
     *
     * <p><b>Note</b>: The result of this method is that same as the
     * result of getEJBObject().getPrimaryKey().
     *
     * @return The EJB object currently associated with the instance.
     *
     * @exception IllegalStateException Thrown if the instance invokes this
     *    method while the instance is in a state that does not allow the
     *    instance to invoke this method.
     */
    Object getPrimaryKey() throws IllegalStateException;
}
