/*
 * $Id: SerializableMethod.java,v 1.7 1999/11/13 22:17:06 harishp Exp $
 *
 * Copyright 1998, 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the proprietary information of Sun Microsystems, Inc.  
 * Use is subject to license terms.
 * 
 */

package javax.ejb.deployment;

import java.lang.reflect.Method;
import java.io.*;

/**
 * The class SerializableMethod is used internally as a serializable 
 * representation of the java.lang.reflect.Method class. This class is
 * transparent to the EJB API.
 *
 * <p> We use a String containing the method's name and parameter signature
 * as the serializable representation of a Method object.
 */
public class SerializableMethod implements Serializable {

    SerializableMethod(Method method)
    {
	super();
	if (method != null) {
	    cls = method.getDeclaringClass();
	    methodSignature = getSignature(method);
	}
    }

    Method getMethod()
	throws NoSuchMethodException
    {
	if (cls == null)
	    return null;
	
	Method[] methods = cls.getMethods();

	for (int i = 0; i < methods.length; i++) {
	    if (getSignature(methods[i]).equals(methodSignature))
		return methods[i];
	}

	/* The local class is incompatible with the deployment descriptor. */
	throw new NoSuchMethodException(cls.getName() + "(" +
					methodSignature + ")");
    }

    private String getSignature(Method method) 
    {
	String rv = method.getName();
	rv += "(";

	Class[] paramTypes = method.getParameterTypes();
	for (int i = 0; i < paramTypes.length; i++) {
	    rv += paramTypes[i].getName();
	    if (i < paramTypes.length - 1)
		rv += ",";
	}
	
	rv += ")";
	return rv;
    }
    
    private Class cls;
    private String methodSignature;
}
