//
//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
package jcs.xml.resultset;

import jcs.util.*;
import jcs.xml.resultset.*;
import jcs.xml.*;

import java.util.Vector;

/**  
 *  A client command line interpreter to invoke
 *  element access and update methods
 *  of the <code>ResultSetXml</code> class.
 *  <p>This command line interpreter invokes the 
 *  <code>getColumn(...)</code> and <code>setColumn(...)</code> 
 *  methods, which reference and modify the columns of the result set.  
 *  Those methods can be invoked in either client or server environments.
 *  This class <code>DoSetGet</code> invokes the 
 *  <code>getColumn(...)</code> and <code>setColumn(...)</code> 
 *  methods in a client environment, using client files for the
 *  input and results.
 */  


public class DoVectorOps {

   /**  
    *  The DoVectorOps class contains only static methods, and cannot be instantiated.
    */    
   private DoVectorOps() {}  

   /**  
    *  Invokes the <code>getColumn(...)</code> and <code>setColumn(...)</code> 
    *  methods of the <code>ResultSetXml</code> class from a client command line. 
    *  <p>Usage:
    *  <pre>
    *  java jcsexec.DoVectorOps -I input-file-name -O output-file-name 
    *       -R row-number {-C column-number | -N column-name} -U update-value 
    *  </pre>
    *  <p>For example assume that <code>emps.xml</code> is a file containint
    *  an XML result set, as produced by <code>RX.query2Xml</code> or
    *  by <code>ResultSetXml</code>.
    *  <p>Example command line:
    *  <pre> 
    *  java jcsexec.DoVectorOps -I emps.xml -O emps-updated.xml
    *         -R 3 -N salary  -U "123.45"
    *  </pre> 
    *  <p>This call will print the current value of the <code>salary</code>
    *  column of row number 3 of the result set in <code>emps.xml</code>,
    *  and store the updated XML document in the file <code>emps-updated.xml</code>.
    *  <p>The parameters tags are as follows.  If the column name 
    *  (<b>-N</b>) parameter is supplied, it is used instead of the column number
    *  (<b>-C</b>) parameter.
    *  <ul>
    *  <li><b>-I</b>:  the name of a client file containing the XML document
    *  <li><b>-O</b>:  the name of a client file for the output script
    *  <li><b>-R</b>:  the number of a row of the result set
    *  <li><b>-C</b>:  the number of a column of the result set
    *  <li><b>-N</b>:  the name of a column of the result set
    *  <li><b>-U</b>:  an update value for the given column of the given row
    *  </ul>
    *  @param args  a string array with the command line arguments    
    */  


   public static void main (String args[]) {
      try{
         String[] argTags = {"-I", "-O", "-R", "-C",  "-N", "-U"};
         String[] argVals = {"", "", "0", "0", "", ""};
         StringUtil.doCommandArgs(args, argTags, argVals);
         String inFile = argVals[0];
         String outFile = argVals[1];
         int rowNumber = Integer.parseInt(argVals[2]); 
         int columnNumber = Integer.parseInt(argVals[3]);
         String columnName = argVals[4];
         String updateValue = argVals[5];  

         if (inFile.equals("") || outFile.equals("")) {
            System.out.println("\nUsage:  java jcsexec.DoVectorOps  "
                    +" -I input-file-name -O output-file-name "
                    + "\n              -R row-number -C column-number [-N column-name] "
                    + "-U update-value \n");
            return;
         }
         
         String xml = FileUtil.file2String(inFile);
         ResultSetXml rsxml = new ResultSetXml(xml);
         String result1 = "";
         String result2 = "";
         String columnID = "";
         if (columnName.equals("")) {     //No column name arg, so use column number 'j'
            result1 = rsxml.getColumn(rowNumber, columnNumber);
            rsxml.setColumn(rowNumber, columnNumber, updateValue);
            if (rsxml!=null) {
               result2 = rsxml.getColumn(rowNumber, columnNumber);            
            } else {
               result2="***update returned null***";
            }
            columnID=""+columnNumber;
         } else {
            result1 = rsxml.getColumn(rowNumber, columnName);
            rsxml.setColumn(rowNumber, columnName, updateValue);
            if (rsxml!=null) {
               result2 = rsxml.getColumn(rowNumber, columnName);            
            } else {
               result2="update returned null";
            }
            columnID=columnName;
         }

         System.out.println("Result of inFile.getColumn (" 
                                  + rowNumber + ", "+columnID+") is '"
              + result1 + "'");

         System.out.println("Performed inFile.setColumn(" + rowNumber + ", " 
                                        + columnID+ ", "+ updateValue+") ");
         System.out.println("   result.getColumn("+ rowNumber + ", "+columnID+") is "
                 + result2 + "\n");
         if (rsxml!=null) {
            FileUtil.string2File(outFile, rsxml.toString());
         }
      } catch (Exception e) {
         System.out.println("Exception: \n"  
           + StringUtil.stackTrace2String(e)  );

      }
   }
}
