//
//  Copyright (c) 1999
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
/**  
 *  A non-instantiable class with miscellaneous static methods
 *  that illustrate the use of Java methods in SQL.
 */  

public class Misc {

   /**  
    *  The Misc class contains only static methods, and cannot be instantiated.
    */     
   private Misc() {}  

   /**  
    *  Removes leading blanks from a String. 
    */  
   public static String stripLeadingBlanks(String s) {
      if (s == null) return null;      
      for (int scan=0; scan<s.length( ); scan++) {
         if ( !java.lang.Character.isWhitespace(s.charAt(scan) )) {
            break;
         } else if (scan == s.length( )) {
            return  "";
         } else {
            return s.substring(scan);
         } 
      }
      return "";
   }
   /**  
    * Extracts the street number from an address line.
    * E.g.  Misc.getNumber("   123 Main Street") == 123
    *    Misc.getNumber("   Main Street")  == 0
    *    Misc.getNumber("") == 0
    *    Misc.getNumber("   123   ") ==  123
    *    Misc.getNumber("   Main 123 ")  == 0
    * @param  s   a string assumed to have address data
    * @return     a string with the extracted street number
    */  

   public static int getNumber (String s) {
      String stripped = stripLeadingBlanks(s);
      if (s==null) return -1;
      for(int right=0; right < stripped.length( ); right++) {
         if ( !java.lang.Character.isDigit(stripped.charAt(right))) {
            break;
         } else if (right == 0) {
            return 0;
         } else {
            return java.lang.Integer.parseInt(stripped.substring(0, right), 10);
         }
      }
      return -1;
   }

   /**  
    * Extracts the "street" from an address line.
    * E.g.  Misc.getStreet("   123 Main Street") == "Main Street"
    *    Misc.getStreet("   Main Street")  == "Main Street"
    *    Misc.getStreet("") ==  ""
    *    Misc.getStreet("   123   ") ==  ""
    *    Misc.getStreet("   Main 123 ")  == "Main 123"
    * @param  s   a string assumed to have address data
    * @return     a string with the extracted street name
    */  
   public static String getStreet(String s) {
      int left;
      if (s==null) return null;
      for (left=0; left<s.length( ); left++) {
         if (java.lang.Character.isLetter(s.charAt(left))) {
            break;
         } else if (left == s.length( )) {
            return "";
         } else {
           return s.substring(left);
         }
      }
      return "";
   }
}

