//
//  Copyright (c) 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//

import java.lang.*;
import java.sql.*;
import java.math.*;

/**  
 *   A class with a <i>main</i> method that calls the SQLJ procedure "rankedemps" 
 *   and prints the returned result set.
 */  

public class CallRankedEmps {

   /**  
    *  The Caller class contains only a static method, and cannot be instantiated.
    */     
   private CallRankedEmps() {}  

   /**  
    *  Invokes the "<i>rankedemps</i>" procedure.
    *  <p>Usage:
    *  <pre>
    *  java CallRankedEmps  url  regionParm
    *  </pre>
    *  <ul>  
    *  <li>The <i>url</i> parameter is the server-name, port-number,
    *  user-name, and password for a JDBC connection.  </li>
    *  <li>The <i>regionParm</i> parameter is an integer region code that 
    *      will be passed as the parameter to <i>rankedemps</i>.</li>
    *  </ul>
    *  <p>Example invocations
    *  <pre>
    *  java CallRankedEmps  "antibes:4000?user=smith&password=xxx" 3
    *  java CallRankedEmps  "antibes:4000?user=sa" 5
    *  </pre>
    *  @param  args  a string array that is the command-line arguments
    */  
    public static void main(String args[]) {

        if (args.length!=2) {
            System.out.println("\n Usage:    " 
                 + "java CallRankedEmps server-name:port-number parmValue ");
            return;
        }

        try{
           int regionParm = Integer.parseInt(args[1]);
           Class.forName("com.sybase.jdbc.SybDriver");
           Connection conn =   DriverManager.getConnection("jdbc:sybase:Tds:"+args[0]);

           /*  The following code illustrates the call of "rankedemps".
            *  Note that this is ordinary code for a call of an SQL stored procedure
            *  that returns a result set.  I.e. the code is unaffected by the fact
            *  that "rankedemps" is an SQLJ procedure.
            */
           java.sql.CallableStatement stmt = conn.prepareCall( "{call rankedemps(?)}");
           stmt.setInt(1, regionParm);
           ResultSet rs = stmt.executeQuery();
           while (rs.next())  {
              String name = rs.getString("name");
              int region = rs.getInt("region");
              BigDecimal sales = rs.getBigDecimal("sales");
              System.out.print(" Name = " + name);
              System.out.print(" Region = " + region);
              System.out.print(" Sales = " + sales);
              System.out.println( );
           }




        }  catch (Exception e) {
           System.out.println("\n Exception: ");
           e.printStackTrace();
        }

    }
}
