--  SQLJ examples
--
--      The example SQLJ statements.
--      

-- *******************************************************

select java.lang.Math.sqrt(16)
go

-- *******************************************************

create function squareroot
        (inputnumber double precision) returns double precision
language java parameter style java
external name 'java.lang.Math.sqrt'
go

-- *******************************************************

select squareroot(16)
go

-- *******************************************************

create table salesemps
	(name varchar(50), id char(5),
	state char(20), sales decimal (6,2),
	jobcode integer null)

go

insert into salesemps values("Adams", "00001", "MN", 123.01, 1)
insert into salesemps values("Baker", "00003", "VT", 323.01, 2)
insert into salesemps values("Carter", "00005", "NH", 523.01, 3)
insert into salesemps values("Davis", "00007", "FL", 723.01, 2)
insert into salesemps values("Earl", "00009", "GA", 923.01, 1)
insert into salesemps values("Foster", "00011", "CA", 1123.01, 2)
insert into salesemps values("George", "00013", "AZ", 3123.01, 3)
insert into salesemps values("Harris", "00015", "NV", 5123.01, 2)
insert into salesemps values("Jackson", "00017", "MN", 7123.01, 1)
insert into salesemps values("Kramer", "00019", "VT", 9123.01, 2)
	
go


-- *******************************************************

select state, SQLJExamples.region(state) as region 
from salesemps
where SQLJExamples.region(state)=3

-- Introduce a mis-spelling:
update salesemps set state="Cal" where state="CA"
go

-- The following now raises an exception due to the mis-spelling:
select name, state, SQLJExamples.region(state) as region 
from salesemps
where SQLJExamples.region(state)=3
go
-- The following call corrects the spelling "Cal" to "CA"
select SQLJExamples.correctStates("Cal", "CA")
go

-- *******************************************************
create function regionof(state char(20)) 
		returns integer
language java parameter style java
external name 'SQLJExamples.region(java.lang.String)'

go

-- *******************************************************

select name, state, regionof(state) as region
	from salesemps
where regionof(state)=3
go

-- *******************************************************

create procedure correctstates(old char(20), 
		notold char(20))
	modifies sql data
	language java parameter style java
	external name
	'SQLJExamples.correctStates (java.lang.String, java.lang.String)'

go

-- *******************************************************

--  Introduce a mis-spelling
update salesemps set state="Geo" where state="GA"

-- Correct the mis-spelling
execute correctstates 'Geo', 'GA'

go

-- *******************************************************

select name,  SQLJExamples.job(jobcode)
from salesemps
where SQLJExamples.job(jobcode) <> "Admin"

go

-- *******************************************************

--  Change a 'jobcode' value to null:
update salesemps set jobcode=null where name = "Baker"
go

-- The following will now raise an exception when the null value
-- is passed as a parameter to "job".
select name,  SQLJExamples.job(jobcode)
from salesemps
where SQLJExamples.job(jobcode) <> "Admin"

-- *******************************************************

select name, 
	case when jobcode is not null 
		then SQLJExamples.job(jobcode)
		else null end
from salesemps 
where 
	case when jobcode is not null 
		then SQLJExamples.job(jobcode)
		else null end <> "Admin"

go

-- *******************************************************

create function jobof(jc integer)
	returns varchar(20)
returns null on null input
language java parameter style java
external name 'SQLJExamples.job(int)'

go

-- *******************************************************

select name, jobof(jobcode)
	from salesemps
where jobof(jobcode) <> "Admin"

go

-- *******************************************************
-- procedure that returns output parameters
--
create procedure best2
   (out n1 varchar(50), out id1 varchar(5),  out s1 decimal(6,2), out r1 integer,
   out n2 varchar(50), out id2 varchar(50), out s2 decimal(6,2),  out r2 integer, 
   in region integer)
   language java parameter style java
   external name 'SQLJExamples.bestTwoEmps \
     (java.lang.String[], java.lang.String[], java.math.BigDecimal[],  int[],  \
     java.lang.String[], java.lang.String[],  java.math.BigDecimal[], int[],  int)'

go

-- *******************************************************


declare @n1 varchar(50), @id1 varchar(5), @s1 decimal (6,2), @r1 integer, 
        @n2 varchar(50), @id2 varchar(50), @r2 integer, @s2 decimal(6,2),
	@region integer
select @region = 0
execute best2 @n1 out, @id1 out, @s1 out, @r1 out, @n2 out, @id2 out,  @s2 out, @r2 out, @region 

go

-- *******************************************************

sp_help best2
go

-- *******************************************************
-- Procedure that returns result sets
--

create procedure rankedemps(region integer)
	dynamic result sets 1
	language java parameter style java
	external name 'SQLJExamples.orderedEmps(int, java.sql.ResultSet[])'

go

exec rankedemps 0
go

-- *******************************************************

drop function squareroot
go
drop table salesemps
go
drop procedure correctstates
go
drop function regionof
go
drop function jobof
go
drop procedure best
go
drop procedure rankedemps
go

-- *******************************************************
