//
//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
package jcs.xml.order;

import jcs.util.*;
import jcs.xml.*;
import jcs.xml.order.*;

import org.w3c.dom.*;

import java.io.*;
import java.sql.*;
import java.util.*;

/**  
 *  A client command line interpreter to demonstrate and test the 
 *  <code>OrderXml</code> class (a subclass of the <code>JXml</code>  
 *  class.
 */  

public class DoOrder{

   /**  
    *  The DoOrder class contains only static methods, and cannot be instantiated.
    */    
   private DoOrder() {}  

   /**  
    *  Interprets the "action" (<b>-A</b>) parameter to perform various
    *  calls of the <code>OrderXml</code> class.
    *  <p>Parameters (not all of the parameters are used with each action):
    *  <ul>
    *  <li><b>-CID</b>:      the CustomerId of the order
    *  <li><b>-CNAME</b>:    the CustomerName of the order
    *  <li><b>-DATE</b>:     the Date of the order
    *  <li><b>-I</b>:        the name of an input file with an Order document
    *  <li><b>-IID</b>:      the ItemId of an item
    *  <li><b>-INAME</b>:    the ItemName of an item
    *  <li><b>-INUM</b>:     the number of an item in the order, beginning with 0
    *  <li><b>-O</b>:        the name of a file for output   
    *  <li><b>-QUANT</b>:    the Quantity of an item in the order
    *  <li><b>-S</b>:        the <code>server</code> identification. See below.
    *  <li><b>-UNIT</b>:     the "unit" attribute of a <Quantity>
    *  </ul>
    *  <p>For a description of the <code>server</code> parameter (<b>-S</b>), 
    *  see the method <code>jcs.util.ExecSql.connectTo</code>.
    *  <p>Actions:
    *  <ul>
    *  <li><b>updateheader</b> - update the Order document contained 
    *  in the input file  (<b>-I</b>) with a new <Date> (<b>-DATE</b>), 
    *  <CustomerId> (<b>-CID</b>) and <CustomerName> (<b>-CNAME</b>).
    *  <p>Example:
    *  <pre> 
    *  java jcs.xml.order.DoOrder -I Order.xml -A updateheader -CID 123       \
    *     -CNAME "Acme Applied" -DATE "99/07/04" -O Order-updateheader.xml
    *  </pre> 
    *  <li><b>updateitem</b> - update the I-th item in the Order document 
    *  contained in the input file (<b>-I</b>)
    *  with a new <ItemId> (<b>-IID</b>), <ItemName> (<b>-INAME</b>),
    *  <ItemNumber> (<b>-INUM</b>), <Quantity> (<b>-QUANT</b>), and
    *  "unit" attribute (<b>-UNIT</b>).
    *  <p>Example:
    *  <pre> 
    *  java jcs.xml.order.DoOrder -I Order.xml -A updateitem -INUM 2     \
    *     -IID 963 -INAME "Curved Cover Caps" -QUANT 17 -UNIT 12 -O Order-updateitem.xml
    *  </pre> 
    *  <li><b>deleteitem</b> -  Delete the I-th item in the Order document
    *   contained in the input file (<b>-I</b>). 
    *  <p>Example:
    *  <pre> 
    *  java jcs.xml.order.DoOrder -I Order.xml -A deleteitem -INUM 2  \
    *        -O Order-updateitem.xml   
    *  </pre> 
    *  <li><b>appenditem</b> - Append a new item to the Order contained 
    *  in the input file (<b>-I</b>), using the <ItemId> (<b>-IID</b>), 
    *  <ItemName> (<b>-INAME</b>),  <ItemNumber> (<b>-INUM</b>), 
    *  <Quantity> (<b>-QUANT</b>), and "unit" attribute (<b>-UNIT</b>).
    *  <p>Example:
    *  <pre> 
    *  java jcs.xml.order.DoOrder -I Order.xml -A appenditem      \
    *     -IID 369 -INAME "Capped Curved Covers" -QUANT 71 -UNIT 12 
    *  </pre> 

    *  <li><b>sql2order</b> - Generate an XML "Order" document from the contents
    *  of specified rows of a set of SQL tables.  This action assumes the following
    *  SQL example tables:
    *  <pre> 
    *  create table customers 
    *     (customer_id varchar(5) not null unique, 
    *     customer_name varchar(50) not null)
    *  create table orders 
    *     (customer_id varchar(5) not null, 
    *  order_date datetime not null, 
    *     item_id varchar(5) not null, 
    *     quantity int not null)
    *  create table items 
    *     (item_id varchar(5) unique, 
    *     item_name varchar(20),
    *     unit smallint)
    *  insert into customers values("123", "Acme Alpha")
    *  
    *  insert into orders values ("123", "2001/05/07", "987", 5)
    *  insert into orders values ("123", "2001/05/07", "654", 3)
    *  insert into orders values ("123", "2001/05/07", "579", 1)
    *  
    *  insert into items values ("987", "Widget", 1)
    *  insert into items values ("654", "Medium Connecter", 12)
    *  insert into items values ("579", "Type 3 clasp", 1)
    *  
    *  </pre> 
    *  <p>Example:
    *  <pre> 
    *  java jcs.xml.order.DoOrder -A sql2order -CID 123 -DATE "99/07/04
    *                         -O order-generated.xml -S server" 
    *  </pre> 
    *  <p>The <b>sql2order</b> action joins the above tables, selects the rows
    *  with the given <code>customer_id</code> and <code>order_date</code>,
    *  and returns an XML "Order" document with that data.
    *  </ul>
    *  @param args  a string array with the command line arguments    
    */  

   public static void main (String args[]) {
             
      String[] argTags 
              = {"-I", "-O", "-A", "-DATE", "-CID", "-CNAME", 
                 "-INUM", "-IID", "-INAME", "-QUANT", "-UNIT",  "-S"};
      String[] argVals = {"", "", "", "", "", "",
                          "0","", "","", "1",  ""};
      try{
         StringUtil.doCommandArgs(args, argTags, argVals);
         String inFile = argVals[0];
         String outFile = argVals[1];
         String action = argVals[2].toLowerCase();
         String date = argVals[3];
         String customerId = argVals[4];
         String customerName = argVals[5];
         int itemNumber = Integer.parseInt(argVals[6]);
         String itemId = argVals[7];
         String itemName = argVals[8];
         String quantity = argVals[9];
         String unit = argVals[10];
         String server = argVals[11];
         String xml = "";
         if (!inFile.equals("")) {
            xml = FileUtil.file2String(inFile);
            if (xml==null || xml.equals("")) {
               System.out.println("\nThe inFile is empty\n");
               return;
            }
         }     

         if (action.equals("updateheader")) {
            OrderXml order = new OrderXml(xml);
            String oldDate = order.getOrderElement("Date");
            String oldCustomerId = order.getOrderElement("CustomerId");
            String oldCustomerName = order.getOrderElement("CustomerName");
            System.out.println("Updated order header:");
            System.out.println("  Old: Date("+oldDate+ ") "
                               +"CustomerId("+oldCustomerId + ") "
                               +"CustomerName("+oldCustomerName + ")");

            order.setOrderElement("Date", date);
            order.setOrderElement("CustomerId", customerId);
            order.setOrderElement("CustomerName", customerName);
            String newDate = order.getOrderElement("Date");
            String newCustomerId = order.getOrderElement("CustomerId");
            String newCustomerName = order.getOrderElement("CustomerName");

            FileUtil.string2File(outFile, order.toString());

            System.out.println("  New: Date("+newDate+ ") "
                               +"CustomerId("+newCustomerId + ") "
                               +"CustomerName("+newCustomerName + ")");

         } else if (action.equals("updateitem")) {
            OrderXml order = new OrderXml(xml);
            String oldItemId = order.getItemElement(itemNumber, "ItemId");
            String oldItemName = order.getItemElement(itemNumber, "ItemName");
            String oldQuantity = order.getItemElement(itemNumber, "Quantity");
            System.out.println("\nUpdated item " + itemNumber +":");
            System.out.println("  Old: ItemId("+oldItemId+ ") "
                               +"ItemName("+oldItemName + ") "
                               +"Quantity("+oldQuantity + ")");
            order.setItemElement(itemNumber, "ItemId", itemId);
            order.setItemElement(itemNumber, "ItemName", itemName);
            order.setItemElement(itemNumber, "Quantity", quantity);
            String newItemId = order.getItemElement(itemNumber, "ItemId");
            String newItemName = order.getItemElement(itemNumber, "ItemName");
            String newQuantity = order.getItemElement(itemNumber, "Quantity");

            FileUtil.string2File(outFile, order.toString());

            System.out.println("  New: ItemId("+newItemId+ ") "
                               +"ItemName("+newItemName + ") "
                               +"Quantity("+newQuantity + ")");
            
         } else if (action.equals("deleteitem"))  {
            OrderXml order = new OrderXml(xml);
            order.deleteItem(itemNumber);
            FileUtil.string2File(outFile, order.toString());
            System.out.println("Deleted item "+itemNumber +"\n");

         } else if (action.equals("appenditem"))  {
            OrderXml order = new OrderXml(xml);
            order.appendItem(itemId, itemName, quantity, unit);
            FileUtil.string2File(outFile, order.toString());
            System.out.println("Appended item: Id: "+itemId+" Name: "+itemName
                 +" Quantity: " + quantity + " Unit: "+unit + "\n");    

         } else if (action.equals("sql2order")) {
            OrderXml newOrder = new OrderXml(date, customerId, server); 
	    if (newOrder==null) {
		throw new Exception ("Null Order");
	    }
            String newDate = newOrder.getOrderElement("Date");
            String newCustomerId = newOrder.getOrderElement("CustomerId");
            String newCustomerName = newOrder.getOrderElement("CustomerName");
            System.out.println("New Order:  Date:"+newDate+" CustomerId: " + newCustomerId
		+ " CustomerName: "+ newCustomerName);
            FileUtil.string2File(outFile, newOrder.toString());

         } else {
            System.out.println("Invalid action: "+ action);
         }
      } catch (Exception e) {
         System.out.println("Do Order exception:");
         e.printStackTrace();
      }
   }



}