//
//  Copyright (c) 1999, 2001
//  Sybase, Inc
//  Emeryville, CA 94608
//  All Rights Reserved
//
package jcs.xml.resultset;

import jcs.util.*;
import jcs.xml.resultset.*;

import java.io.*;
import java.sql.*;
import java.util.*;

/**  
 *  
 *  A client command line shell to demonstrate uses of 
 *  the <code>ResultSetVector</code> class. 
 */  

public class  Do2Vector implements java.io.Serializable  {


   /**  
    *  The Do2Vector class contains only static methods, and cannot be instantiated.
    */    
   private Do2Vector() {}  

   /**  
    *  Invokes the constructors and methods of the 
    *  <code>resultSetVector</code> class from a client command line.
    *  The purpose of these calls is to demonstrate or test the
    *  <code>resultSetVector</code> class.
    *  <p>Usage:
    *  <pre>
    *  java jcsexec.Do2Vector  -I inFile  -O outFile
    *
    *  java jcsexec.Do2Vector -Q query -S server -O outFile
    *  </pre>
    *  <p>For example, assume that <code>emps.xml</code> is a file containing
    *  an XML result set, as produced by <code>RX.query2Xml</code> or
    *  by <code>ResultSetXml</code>.
    *  <p>Example command line:
    *  <pre> 
    *  java jcsexec.Do2Vector -I emps.xml  -O emps-vector-out.xml
    *  
    *  java jcsexec.Do2Vector -Q "select * from emps"      \
    *       -S "antibes:4000?user=sa" -O emps-vector-out2.xml
    *  </pre> 
    *  <p>The first of these calls will translate the XML result set 
    *  contained in <code>emps.xml</code> to a Java vector, and then copy that
    *  vector into file <code>emps-vector-out.xml</code>.
    *  The second call will translate the SQL result set from the given query
    *  (<b>-Q</b>) to a Java vector, and then coppy that vector into file
    *  <code>emps-vector-out2.xml</code>.
    *  <p>The parameter tags are as follows.    For a detailed description
    *  of the <code>query</code> parameter, see the <code>ResultSetVector</code>
    *  constructur, which this command-line routine invokes. 
    *  For a description of the <code>server</code> parameter, 
    *  method <code>jcs.mics.ExecSql.connectTo</code>.
    *  <ul>
    *  <li><b>-I</b>:  the name of a client file containing the XML document
    *  <li><b>-O</b>:  the name of a client file for the output display
    *  <li><b>-Q</b>:  a string that is an SQL query returning a result set
    *  <li><b>-S</b>:  the <code>server</code> parameter
    *  </ul>
    *  <p>Depending on whether the file (<b>-I</b>) or query (<b>-Q</b>) parameter
    *  is supplied, the method will invoke the appropriate constructor method
    *  of the <code>ResultSetVector</code> class.  The action of these methods
    *  is either:
    *  <ul>
    *  <li>Translate a <code>ResultSetXml</code> document to a
    *  <code>ResultSetVector</code> whose <code>content</code> field
    *  is a Java Vector containing the rows and columns of the 
    *  result set.
    *  <li>Evaluate an SQL query, and translate its result set to a
    *  <code>ResultSetVector</code> whose <code>content</code> field
    *  is a Java Vector containing the rows and columns of the 
    *  result set.
    *  </ul>    
    *  @param args  a string array with the command line arguments    
    */  


   public static void main (String args[]) {      
      try{      
         String[] argTags = {"-I", "-O", "-Q", "-S"};
         String[] argVals = {"", "", "", ""};
         StringUtil.doCommandArgs(args, argTags, argVals);
         String inFile = argVals[0];
         String outFile = argVals[1];
         String query = argVals[2];
         String server = argVals[3];
         String xmlFile = "";
         ResultSetVector rsv = null;
         if (outFile.equals("") 
              || ( inFile.equals("") && query.equals(""))
              || (!query.equals("") && server.equals("")) ) {
            System.out.println("\nUsage:  java Do2Vector "
                    +" -I input-file-name  -O output-file-name  \n"
                  + "or java Do2Vector "
                    +" -Q sql-query -S server-name -O output-file-name  \n"  );
            return;
         }
         
         if (query.equals("")) {
            xmlFile = FileUtil.file2String(inFile);
            rsv = new ResultSetXml(xmlFile).toResultSetVector();
         } else {
            rsv = new ResultSetVector(query, server);
         }
         String tableString = "";
         String rowString = "";
 
         for (int i=0; i<(rsv==null?0:rsv.content().size()); i++)  {
            Vector rowVector = (Vector)rsv.content().elementAt(i);
            for (int j=0; j<rowVector.size(); j++)  {
               rowString += " "+(String)rowVector.elementAt(j);
            }
            tableString += "\n"+rowString;
            rowString ="";
         }
/*
         tableString += "\n ============rev.rowCount()";
         tableString += "\n"+ rsv.rowCount();

         tableString += "\n ============rsv.columnCount()";
         tableString += "\n"+ rsv.columnCount();

         tableString += "\n ============rsv.getColumn(0,0)";
         tableString += "\n"+ rsv.getColumn(0,0);

         tableString += "\n ============rsv.getColumn(1,1)";
         tableString += "\n"+ rsv.getColumn(1,1);

         tableString += "\n ============rsv.toStringRow(0):";
         tableString += "\n"+ rsv.toStringRow(0, " : ");

         tableString += "\n ============rsv.toString():";
         tableString += "\n"+ rsv.toStringColumn(1, " - ");

         tableString += "\n\n\n ========rsv.someString(3, =, 77): true";
         tableString += "\n                                        result: "
                            + rsv.someString(3, "=", "77");

         tableString += "\n\n\n =======rsv.someString(3, ==, 77): true";
         tableString += "\n                                        result: "
                            + rsv.someString(3, "==", "77");

         tableString += "\n\n\n ======rsv.someString(3, <>, 77): true";
         tableString += "\n                                        result: "
                            + rsv.someString(3, "<>", "77");

         tableString += "\n\n\n ==========rsv.someString(3, !=, 77): true";
         tableString += "\n                                        result: "
                            + rsv.someString(3, "!=", "77");

         tableString += "\n\n\n ===========rsv.someString(3, =, 76): null";
         tableString += "\n                                        result: "
                            + rsv.someString(3, "=", "76");

         tableString += "\n\n\n == ========rsv.someString(0, ==, 444): false";
         tableString += "\n                                        result: "
                            + rsv.someString(0, "==", "444");

         tableString += "\n\n ============rsv.someString(3, >, 98): true";
         tableString += "\n                                        result: "
                             + rsv.someString(3, ">", "98");

         tableString += "\n\n ============rsv.someString(3, >=, 99): true";
         tableString += "\n                                        result: "
                             + rsv.someString(3, ">=", "99");

         tableString += "\n\n ============rsv.someString(3, >, 99): null";
         tableString += "\n                                        result: "
                             + rsv.someString(3, ">", "99");

         tableString += "\n\n ===========rsv.someString(0, >=, 987): true";
         tableString += "\n                                        result: "
                             + rsv.someString(0, ">=", "987");


         tableString += "\n\n ==========rsv.someString(0, >, 987): false";
         tableString += "\n                                        result: "
                             + rsv.someString(0, ">", "987");


         tableString += "\n\n ====================rsv.someString(3, <, 67): true";
         tableString += "\n                                        result: "
                             + rsv.someString(3, "<", "67");

         tableString += "\n\n ====================rsv.someString(3, <, 66): true";
         tableString += "\n                                        result: "
                             + rsv.someString(3, "<", "66");

         tableString += "\n\n ====================rsv.someString(3, =, ''): true";
         tableString += "\n                                        result: "
                             + rsv.someString(3, "=", "");

         tableString += "\n\n ====================rsv.someString(0, <, 125): true";
         tableString += "\n                                        result: "
                               + rsv.someString(0, "<", "125");


         tableString += "\n\n ====================rsv.someString(0, <=, 123): true";
         tableString += "\n                                        result: "
                               + rsv.someString(0, "<=", "123");


         tableString += "\n\n ====================rsv.someString(0, <, 100): false";
         tableString += "\n                                        result: "
                               + rsv.someString(0, "<", "100");


         tableString += "\n\n ================rsv.someString(2, <, 123.45): null";
         tableString += "\n                                        result: "
                               + rsv.someString(2, "<", "123.45");


         tableString += "\n\n ================rsv.someString(2, =, 456.00): null";
         tableString += "\n                                        result: "
                               + rsv.someString(2, "=", "456.00");


         //   ALL -----------------------------------------------------

         tableString += "\n\n\n ===========rsv.allString(0, =, 333): false";
         tableString += "\n                                        result: "
                            + rsv.allString(0, "=", "333");

         tableString += "\n\n\n ===========rsv.allString(0, ==, 333): false";
         tableString += "\n                                        result: "
                            + rsv.allString(0, "==", "333");

         tableString += "\n\n\n ===========rsv.allString(0, !=, 333): true";
         tableString += "\n                                        result: "
                            + rsv.allString(0, "!=", "333");

         tableString += "\n\n\n ===========rsv.allString(0, <>, 333): true";
         tableString += "\n                                        result: "
                            + rsv.allString(0, "<>", "333");


         tableString += "\n\n\n ===========rsv.allString(0, >=, 123): true";
         tableString += "\n                                        result: "
                            + rsv.allString(0, ">=", "123");


         tableString += "\n\n\n ===========rsv.allString(0, >, 122): true";
         tableString += "\n                                        result: "
                            + rsv.allString(0, ">", "122");

         tableString += "\n\n\n ===========rsv.allString(0, >, 124): false";
         tableString += "\n                                        result: "
                            + rsv.allString(0, ">", "124");


         tableString += "\n\n\n ===========rsv.allString(0, <, 988): true";
         tableString += "\n                                        result: "
                            + rsv.allString(0, "<", "988");

         tableString += "\n\n\n ===========rsv.allString(0, <=, 987): true";
         tableString += "\n                                        result: "
                            + rsv.allString(0, "<=", "987");


         tableString += "\n\n\n ===========rsv.allString(0, <, 987): false";
         tableString += "\n                                        result: "
                            + rsv.allString(0, "<", "987");


         tableString += "\n\n\n ===========rsv.allString(2, >, 111.11): null";
         tableString += "\n                                        result: "
                            + rsv.allString(2, ">", "111.11");

         tableString += "\n\n\n ===========rsv.allString(2, >=, 123.45): null";
         tableString += "\n                                        result: "
                            + rsv.allString(2, ">=", "123.45");

         tableString += "\n\n\n ===========rsv.allString(2, <, 999.99): null";
         tableString += "\n                                        result: "
                            + rsv.allString(2, "<", "999.99");

         tableString += "\n\n\n ===========rsv.allString(2, <=, 999.99): null";
         tableString += "\n                                        result: "
                            + rsv.allString(2, "<=", "999.99");


         tableString += "\n\n\n ===========rsv.allString(2, <> 333.33): null";
         tableString += "\n                                        result: "
                            + rsv.allString(3, "<>", "333.33");

         tableString += "\n\n\n ===========rsv.allString(2, !=, 333.33): null";
         tableString += "\n                                        result: "
                            + rsv.allString(3, "!=", "333.33");

         tableString += "\n\n\n ===========rsv.allString(0, ==, 333.33): false";
         tableString += "\n                                        result: "
                            + rsv.allString(0, "==", "333.33");



*/
         FileUtil.string2File(outFile, tableString);


      } catch (IllegalArgumentException iae) {
         System.out.println("\nInvalid arguments: " + iae.getMessage() + "\n" 
            + "Usage:  java Do2Vector -I file-name -O file-name "
            + "[-T table-name] [-X column-prefix]\n");
            
      } catch (FileNotFoundException notFound) {
            System.err.println("\n Error: File not found (" 
                                  + notFound.getMessage()+ ")\n");
            return;
     
      } catch (Exception e) {
         System.out.println("Exception: \n"  
           + StringUtil.stackTrace2String(e)  );
      }
   }
}
