/*
//---------------------------------------------------------------------
//          Sybase Monitor
//          Confidential Property of Sybase, Inc.
//          (c) Copyright Sybase, Inc 1995,1996
//          All rights reserved
//-----------------------------------------------------------------
//
//  FILE: testhist.c
//
//  Sample program that will connect to a Historical Server and initiate a
//  recording session.
//
*/

/*
///////////////////////////////////////////////////////////////////////////////
//
//  Declarations
//
//
// standard includes
*/
/* stdio.h provides definition for sprintf() */
#include <stdio.h>
#include <string.h>
/*
// local includes
*/
#include "example.h"
#include "mcpublic.h"

/* The following logic for determining whether function prototypes are supported
   is borrowed from the Sybase header file csconfig.h:
*/

#if defined(_MSC_VER)
#define CS_FORCE_PROTOTYPES        1
#endif

#if (__STDC__) || defined(CS_FORCE_PROTOTYPES)
# define _SMC_PROTOTYPES
#endif

#ifdef _SMC_PROTOTYPES
#define _SMC_PROTO(arglist) arglist
#else
#define _SMC_PROTO(ARGLIST) ()
#endif


/*
///////////////////////////////////////////////////////////////////////////////
//
//  Global Variables
*/
#define MAX_BUFFER_SIZE 512

SMC_SERVER_MODE           serverMode = SMC_SERVER_M_HISTORICAL;
SMC_SIZET                 winNum = 2;
SMC_SIZET                 refreshInterval = 5;
SMC_BOOL                  useHardCode = FALSE;
SMC_BOOL                  testFiltering = TRUE;
SMC_BOOL                  testAlarms = TRUE;
SMC_CHAR                  serverName[31];
SMC_CHAR                  monserverName[31];
SMC_CHAR                  historicalServerName[31];
SMC_CHAR                  userName[31];
SMC_CHAR                  userPassword[31];
SMC_CHAR                  interfacesFile[257];
SMC_SIZET                 version = 1001;


SMC_VOID
Cleanup _SMC_PROTO((
  SMC_CONNECT_ID          id,
  SMC_SIZET               numViews,
  SMC_DATAITEM_STRUCT*    ptrDIs,
  SMC_SIZET*              ptrViewSizes,
  SMC_SIZET*              ptrViewIndices,
  SMC_VIEW_ID*            ptrViewIDs,
  SMC_VALUE_UNIONP        ptrData,
  SMC_CHAR*               ptrUserMsg
  ));

SMC_RETURN_CODE
CreateFilters _SMC_PROTO((
  SMC_CONNECT_ID          id,
  SMC_VIEW_ID*            ptrViewIDs,
  SMC_SIZET               winNum
  ));

SMC_RETURN_CODE
CreateAlarms _SMC_PROTO((
  SMC_CONNECT_ID          id,
  SMC_VIEW_ID*            ptrViewIDs,
  SMC_SIZET               winNum
  ));

SMC_RETURN_CODE
CreateViews _SMC_PROTO((
  SMC_CONNECT_ID          id,
  SMC_SIZET               numViews,
  SMC_DATAITEM_STRUCT*    ptrDIs,
  SMC_SIZET*              ptrViewSizes,
  SMC_SIZET*              ptrViewIndices,
  SMC_VIEW_ID*            ptrViewIDs
  ));

SMC_VOID
ErrorCallback _SMC_PROTO((
  SMC_CONNECT_ID          id,
  SMC_COMMAND_ID          commandId,
  SMC_VOIDP               userData
  ));

SMC_RETURN_CODE
Setup _SMC_PROTO((
  SMC_SIZET               winNum,
  SMC_SIZET*              numViews,
  SMC_DATAITEM_STRUCT**   ptrDIs,
  SMC_SIZET**             ptrViewSizes,
  SMC_SIZET**             ptrViewIndices,
  SMC_VIEW_ID**           ptrViewIDs,
  SMC_VALUE_UNIONP*       ptrData,
  SMC_CHAR**              ptrUserMsg
  ));

SMC_VOID
ShowViews _SMC_PROTO((
  SMC_SIZET               numViews,
  SMC_DATAITEM_STRUCT*    ptrDIs,
  SMC_SIZET*              ptrViewSizes,
  SMC_SIZET*              ptrViewIndices
  ));

SMC_INT
main _SMC_PROTO((
  SMC_INT     argc,
  SMC_CHARP   argv[]
  ));


/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: main()
*/
SMC_INT
main(
#ifdef _SMC_PROTOTYPES
  SMC_INT     argc,
  SMC_CHARP   argv[]
  )
#else /* _SMC_PROTOTYPES */
  argc,
  argv
  )
  SMC_INT     argc;
  SMC_CHARP   argv[];
#endif /* _SMC_PROTOTYPES */

{
  SMC_INT i;

  SMC_RETURN_CODE        ret;
  SMC_CONNECT_ID         id = 0;
  SMC_CONNECT_ID*        ptrID = &id;
  SMC_SESSION_ID         sessId = 0;
  SMC_SESSION_ID*        ptrSessID = &sessId;
  SMC_SIZET              numViews = 0;
  SMC_DATAITEM_STRUCT*   ptrDIs = NULL;
  SMC_SIZET*             ptrViewSizes = NULL;
  SMC_SIZET*             ptrViewIndices = NULL;
  SMC_VIEW_ID*           ptrViewIDs = NULL;
  SMC_VALUE_UNIONP       ptrData = NULL;
  SMC_CHAR*              ptrUserMsg = NULL;

  SMC_VALUE_UNION        prop;

  printf("**************************************************************\n");
  printf("******  C-language Test Driver for Monitor Client API  *******\n");
  printf("**************************************************************\n");

  if (argc > 1)
  {
    for (i=1; i<argc; i++)
    {
      printf("Unexpected command line argument: %s\n", argv[i]);
    }
  }


  strcpy(historicalServerName, EX_HIST_SERVER);

  strcpy(monserverName, EX_MON_SERVER);

  strcpy(userName, EX_USERNAME);

  strcpy(userPassword, EX_PASSWORD);

  strcpy(interfacesFile, EX_INTERFACES);


  printf("*********    Testing smc_connect_alloc()   **********\n");
  ret = smc_connect_alloc(ErrorCallback,ptrID);
  if (ret != SMC_RET_SUCCESS)
  {
    printf("ERR: %d returned by smc_connect_alloc()", ret);
    return ((int)SMC_RET_FAILURE);
  }
  printf("******    Testing smc_connect_props() : serverMode   ******\n");
  prop.intValue = serverMode;
  ret = smc_connect_props(id, SMC_PROP_ACT_SET, SMC_PROP_SERVERMODE, &prop,
                          SMC_UNUSED, NULL);
  if (ret != SMC_RET_SUCCESS)
  {
    printf("ERR: %d returned by smc_connect_props(SERVERMODE)", ret);
    return ((int)SMC_RET_FAILURE);
  }
 
  printf("******    Testing smc_connect_props() : serverName   ******\n");
  prop.stringValue = historicalServerName;
  ret = smc_connect_props(id, SMC_PROP_ACT_SET, SMC_PROP_SERVERNAME, &prop,
                          SMC_NULLTERM, NULL);
  if (ret != SMC_RET_SUCCESS)
  {
    printf("ERR: %d returned by smc_connect_props(SERVERNAME)", ret);
    return ((int)SMC_RET_FAILURE);
  }
 
  printf("******    Testing smc_connect_props() : userName   ******\n");
  prop.stringValue = userName;
  ret = smc_connect_props(id, SMC_PROP_ACT_SET, SMC_PROP_USERNAME, &prop,
                          SMC_NULLTERM, NULL);
  if (ret != SMC_RET_SUCCESS)
  {
    printf("ERR: %d returned by smc_connect_props(USERNAME)", ret);
    return ((int)SMC_RET_FAILURE);
  }
 
  printf("******    Testing smc_connect_props() : userPassword    ******\n");
  prop.stringValue = userPassword;
  ret = smc_connect_props(id, SMC_PROP_ACT_SET, SMC_PROP_PASSWORD, &prop,
                          SMC_NULLTERM, NULL);
  if (ret != SMC_RET_SUCCESS)
  {
    printf("ERR: %d returned by smc_connect_props(PASSWORD)", ret);
    return ((int)SMC_RET_FAILURE);
  }

  printf("******    Testing smc_connect_props() : interfacesFile  *****\n");
  prop.stringValue = interfacesFile;
  ret = smc_connect_props(id, SMC_PROP_ACT_SET, SMC_PROP_IFILE, &prop,
                          SMC_NULLTERM, NULL);
  if (ret != SMC_RET_SUCCESS)
  {
    printf("ERR: %d returned by smc_connect_props(PASSWORD)", (int) ret );
    return SMC_RET_FAILURE;
  }
 
 
  printf("******    Testing smc_connect_props() : ErrorCallback    ******\n");
  prop.voidpValue = (SMC_VOIDP) ErrorCallback;
  ret = smc_connect_props(id, SMC_PROP_ACT_SET, SMC_PROP_ERROR_CALLBACK, &prop,
                          SMC_UNUSED, NULL);
  if (ret != SMC_RET_SUCCESS)
  {
    printf("ERR: %d returned by smc_connect_props(ERRORCALLBACK)", ret);
    return ((int)SMC_RET_FAILURE);
  }
 
 
  printf("********     Testing smc_connect_ex()    ************\n");
  ret = smc_connect_ex(id);
  if (ret != SMC_RET_SUCCESS)
  {
    printf("ERR: %d returned by smc_connect_ex()", ret);
    return ((int)SMC_RET_FAILURE);
  }
 

    printf("*********   Testing smc_create_recording_session()   *********\n");
    ret = smc_create_recording_session(
                                       id,
                                       monserverName,
                                       refreshInterval,
                                       NULL,
                                       NULL,
                                       NULL,
                                       SMC_HS_SESS_PROT_PRIVATE,
                                       SMC_HS_SESS_ERR_CONT,
									   SMC_HS_SESS_SCRIPT_NONE,
                                       ptrSessID
                                      );
    if ( ret != SMC_RET_SUCCESS )
    {
      printf("ERR: %d  returned by smc_create_recording_session\n", ret);
      return ((int)SMC_RET_FAILURE);
    }
    printf("Session Id: %d\n", sessId);

  ret = Setup(
              winNum,
              &numViews,
              &ptrDIs,
              &ptrViewSizes,
              &ptrViewIndices,
              &ptrViewIDs,
              &ptrData,
              &ptrUserMsg
             ); 
  if ( ret != SMC_RET_SUCCESS )
  {
    printf("ERR: %d returned by Setup()\n", ret);
    Cleanup(id, numViews, ptrDIs, ptrViewSizes, ptrViewIndices, ptrViewIDs, ptrData, ptrUserMsg);  
    return ((int)SMC_RET_FAILURE);
  }
            

  ShowViews(
            numViews,
            ptrDIs,
            ptrViewSizes,
            ptrViewIndices
           );


  ret = CreateViews(
                    id,
                    numViews,
                    ptrDIs,
                    ptrViewSizes,
                    ptrViewIndices,
                    ptrViewIDs
                   );
  if ( ret != SMC_RET_SUCCESS )
  {
    printf("ERR: %d returned by CreateViews()\n", ret);
    Cleanup(id, numViews, ptrDIs, ptrViewSizes, ptrViewIndices, ptrViewIDs, ptrData, ptrUserMsg);  
    return ((int)SMC_RET_FAILURE);
  }

  if (testFiltering)
  {
     CreateFilters(id, ptrViewIDs, winNum);
  }

  if (testAlarms)
  {
     CreateAlarms(id, ptrViewIDs, winNum);
  }

  /* historical mode */
    printf("**********   Testing smc_initiate_recording()   ***********\n");
    ret = smc_initiate_recording(
                                 id
                                );
    if ( ret != SMC_RET_SUCCESS )
    {
      printf("ERR: %d returned by smc_initiate_recording()\n", ret);
      return ((int)SMC_RET_FAILURE);
    }

  /* drop views and destroy local arrays */
  Cleanup(id, numViews, ptrDIs, ptrViewSizes, ptrViewIndices, ptrViewIDs, ptrData, ptrUserMsg);  


    printf( "*****   Testing   smc_close()          *****\n" );
    ret = smc_close(id, SMC_CLOSE_REQUEST);
    if ( ret != SMC_RET_SUCCESS )
    {
      printf("ERR: %d returned by smc_close()", (int)ret );
      return ((int)SMC_RET_FAILURE);
    }
 
    printf( "*****   Testing   smc_connect_drop()   *****\n" );
    ret = smc_connect_drop(id);
    if ( ret != SMC_RET_SUCCESS )
    {
      printf("ERR: %d returned by smc_connect_drop()", (int)ret );
      return ((int)SMC_RET_FAILURE);
    }
 

  printf("**********************   End of Test   ***********************\n");

  return ((int)SMC_RET_SUCCESS);
}


/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: Cleanup() 
*/
SMC_VOID
Cleanup(
#ifdef _SMC_PROTOTYPES
  SMC_CONNECT_ID          id,
  SMC_SIZET               numViews,
  SMC_DATAITEM_STRUCT*    ptrDIs,
  SMC_SIZET*              ptrViewSizes,
  SMC_SIZET*              ptrViewIndices,
  SMC_VIEW_ID*            ptrViewIDs,
  SMC_VALUE_UNIONP        ptrData,
  SMC_CHAR*               ptrUserMsg
  )
#else /* _SMC_PROTOTYPES */
  id,
  numViews,
  ptrDIs,
  ptrViewSizes,
  ptrViewIndices,
  ptrViewIDs,
  ptrData,
  ptrUserMsg
  )
  SMC_CONNECT_ID          id;
  SMC_SIZET               numViews;
  SMC_DATAITEM_STRUCT*    ptrDIs;
  SMC_SIZET*              ptrViewSizes;
  SMC_SIZET*              ptrViewIndices;
  SMC_VIEW_ID*            ptrViewIDs;
  SMC_VALUE_UNIONP        ptrData;
  SMC_CHAR*               ptrUserMsg;
#endif /* _SMC_PROTOTYPES */
{
  if (ptrViewIDs != NULL) 
  {
    free((char *)ptrViewIDs);
  }

  if (ptrDIs != NULL)
  {
    free((char *)ptrDIs);
  }

  if (ptrViewSizes != NULL)
  {
    free((char *)ptrViewSizes);
  }

  if (ptrViewIndices != NULL)
  {
    free((char *)ptrViewIndices);
  }

  if (ptrData != NULL)
  {
    free((char *)ptrData);
  }

  if (ptrUserMsg != NULL)
  {
    free((char *)ptrUserMsg);
  }

  return;
}


/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: CreateAlarms()
*/
SMC_RETURN_CODE
CreateAlarms( 
#ifdef _SMC_PROTOTYPES
  SMC_CONNECT_ID  client_id,
  SMC_VIEW_ID*    ptrViewIDs,
  SMC_SIZET       winNum
  )
#else /* _SMC_PROTOTYPES */
  client_id,
  ptrViewIDs,
  winNum
  )
  SMC_CONNECT_ID  client_id;
  SMC_VIEW_ID*    ptrViewIDs;
  SMC_SIZET       winNum;
#endif /* _SMC_PROTOTYPES */
{
  SMC_RETURN_CODE               ret;
  SMC_DATAITEM_STRUCT           di;
  SMC_ALARM_ACTION_TYPE         alarmAction;
  SMC_CHAR                      alarmActionData[257];
  SMC_VALUE_UNION               alarmValue;
  SMC_DATAITEM_TYPE             alarmDatatype;
  SMC_ALARM_ID                  alarmId;
  SMC_ALARM_IDP                 ptrAlarmId = &alarmId;
  SMC_VIEW_ID                   viewId;

  if ( winNum != 2 )
  {
    printf(" Alarms not supported for Window: %d in this sample program.\n", winNum);
    return SMC_RET_FAILURE;
  }

  switch (winNum)
  {
    case 2:
      printf(" Setting alarms for Window 2 on DataItem: SMC_NAME_DEV_IO\n");
      di.dataItemName =  SMC_NAME_DEV_IO;
      di.dataItemStatType = SMC_STAT_RATE_SAMPLE;
      strcpy(alarmActionData, "/tmp/alarm.log");
      alarmAction = SMC_ALARM_A_LOG_TO_FILE;
      alarmDatatype = SMC_DI_TYPE_DOUBLE;
      viewId = ptrViewIDs[1];
      break;
  }

/* Setting Alarm to be at 20.3 */

  alarmValue.doubleValue = 20.3;

  printf("************   Testing smc_create_alarm()   *************\n");

  ret = smc_create_alarm_ex(
                          client_id,          /* connectionID           */
                          viewId,             /* viewid                 */
                          &di,                /* distruct               */
                          &alarmValue,        /* alarm value            */
                          alarmDatatype,      /* alarm datatype         */
                          alarmAction,        /* alarm action           */
                          alarmActionData,    /* log file name          */
                          NULL,               /* user-supplied data     */
                          NULL,               /* callback function      */
                          ptrAlarmId          /* alarmIdHandle          */
                         );
  if ( ret != SMC_RET_SUCCESS )
  {
    printf("ERR: %d returned by smc_create_alarm_ex()\n", ret);
    return SMC_RET_FAILURE;
  }

  return SMC_RET_SUCCESS;
}
  

/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: CreateFilters()
*/
SMC_RETURN_CODE
CreateFilters( 
#ifdef _SMC_PROTOTYPES
  SMC_CONNECT_ID client_id,
  SMC_VIEW_ID*   ptrViewIDs,
  SMC_SIZET      winNum
  )
#else /* _SMC_PROTOTYPES */
  client_id,
  ptrViewIDs,
  winNum
  )
  SMC_CONNECT_ID client_id;
  SMC_VIEW_ID*   ptrViewIDs;
  SMC_SIZET      winNum;
#endif /* _SMC_PROTOTYPES */
{
  SMC_RETURN_CODE               ret;
  SMC_DATAITEM_STRUCT           di;
  SMC_VALUE_UNION               filterVal;
  SMC_VALUE_UNIONP              ptrFilterVal = &filterVal;
  SMC_FILTER_ID                 filterId;
  SMC_FILTER_IDP                ptrFilterId = &filterId;
  SMC_VIEW_ID                   viewId;

  if ( winNum != 2 )
  {
    printf(" Filtering not supported for Window: %d\n", winNum);
    return SMC_RET_FAILURE;
  }


  switch (winNum)
  {
    case 2:
      printf(" Filtering Window 2 on DataItem: SMC_NAME_DEV_NAME \n");
      di.dataItemName =  SMC_NAME_DEV_NAME;
      di.dataItemStatType = SMC_STAT_VALUE_SAMPLE;
      viewId = ptrViewIDs[1];
      break;
  }


/* setting Filter string to be the master device: */
  filterVal.stringValue = (char*)("master");

  printf("************   Testing smc_create_filter()   *************\n");

  ret = smc_create_filter(
                          client_id,          /* connectionID           */
                          viewId,             /* viewid                 */
                          &di,                /* distruct               */
                          SMC_FILT_T_EQ,      /* filterType             */
                          ptrFilterVal,       /* filterValueListHandle  */
                          1,                  /* filterValueListLength  */
                          SMC_DI_TYPE_CHARP,  /* filterDataType         */
                          ptrFilterId         /* filterIdHandle         */
                         );
  if ( ret != SMC_RET_SUCCESS )
  {
    printf("ERR: %d returned by smc_create_filter()\n", ret);
    return SMC_RET_FAILURE;
  }

  return SMC_RET_SUCCESS;
}
  

/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: CreateViews() 
*/
SMC_RETURN_CODE
CreateViews( 
#ifdef _SMC_PROTOTYPES
  SMC_CONNECT_ID          id,
  SMC_SIZET               numViews,
  SMC_DATAITEM_STRUCT*    ptrDIs,
  SMC_SIZET*              ptrViewSizes,
  SMC_SIZET*              ptrViewIndices,
  SMC_VIEW_ID*            ptrViewIDs
  )
#else /* _SMC_PROTOTYPES */
  id,
  numViews,
  ptrDIs,
  ptrViewSizes,
  ptrViewIndices,
  ptrViewIDs
  )
  SMC_CONNECT_ID          id;
  SMC_SIZET               numViews;
  SMC_DATAITEM_STRUCT*    ptrDIs;
  SMC_SIZET*              ptrViewSizes;
  SMC_SIZET*              ptrViewIndices;
  SMC_VIEW_ID*            ptrViewIDs;
#endif /* _SMC_PROTOTYPES */
{
  SMC_RETURN_CODE         ret;
  SMC_SIZET               i;
  SMC_CHAR                viewName[11];
  
  for(i=0; i<numViews; i++)
  {
      sprintf(viewName, "view%d", i);
      ret = smc_create_view(
                            id,
                            &(ptrDIs[ (ptrViewIndices[i]) ]),
                            ptrViewSizes[i],
                            viewName,
                            &(ptrViewIDs[i])
                           );
      if ( ret != SMC_RET_SUCCESS )
      {
           printf("ERR: %d returned by smc_create_view\n", ret);
           return SMC_RET_FAILURE;
      }
      printf("=== View Id: %d ===\n", ptrViewIDs[i]);
  }

  return SMC_RET_SUCCESS;
}


/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: ErrorCallback()
*/
SMC_VOID
ErrorCallback(
#ifdef _SMC_PROTOTYPES
  SMC_CONNECT_ID id,
  SMC_COMMAND_ID commandId,
  SMC_VOIDP      userData
  )
#else /* _SMC_PROTOTYPES */
  id,
  commandId,
  userData
)
  SMC_CONNECT_ID id;
  SMC_COMMAND_ID commandId;
  SMC_VOIDP      userData;
#endif /* _SMC_PROTOTYPES */
{
  SMC_VALUE_UNION info;
 
  printf("Inside ErrorCallback()\n");
 
  printf("id = %d\n", id);
  printf("commandId = %d\n", commandId);
  if (userData != NULL)
    printf("userDataHandle = %s", userData);
 
  if (smc_get_command_info(id, commandId, SMC_INFO_ERR_MSG, &info,
                NULL) == SMC_RET_SUCCESS)
    printf("errMsg = %s", info.stringValue );
 
  if (smc_get_command_info(id, commandId, SMC_INFO_ERR_NUM, &info,
                NULL) == SMC_RET_SUCCESS)
    printf("errNum = %d", info.sizetValue );
 
  if (smc_get_command_info(id, commandId, SMC_INFO_ERR_SEVERITY, &info,
                NULL) == SMC_RET_SUCCESS)
    printf("errSeverity = %d", info.sizetValue );
 
  if (smc_get_command_info(id, commandId, SMC_INFO_ERR_MAPSEVERITY, &info,
                NULL) == SMC_RET_SUCCESS)
    printf("mapSeverity = %d", info.sizetValue );
 
  if (smc_get_command_info(id, commandId, SMC_INFO_ERR_SOURCE, &info,
                NULL) == SMC_RET_SUCCESS)
    printf("errSource = %d", info.sizetValue );
 
  if (smc_get_command_info(id, commandId, SMC_INFO_ERR_STATE, &info,
                NULL) == SMC_RET_SUCCESS)
    printf("errState = %d", info.sizetValue );
 
  return;
}


/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: Setup() 
*/
SMC_RETURN_CODE
Setup(
#ifdef _SMC_PROTOTYPES
  SMC_SIZET               winNum,
  SMC_SIZET*              numViews,
  SMC_DATAITEM_STRUCT**   ptrDIs,
  SMC_SIZET**             ptrViewSizes,
  SMC_SIZET**             ptrViewIndices,
  SMC_VIEW_ID**           ptrViewIDs,
  SMC_VALUE_UNIONP*       ptrData,
  SMC_CHAR**              ptrUserMsg
  )
#else /* _SMC_PROTOTYPES */
  winNum,
  numViews,
  ptrDIs,
  ptrViewSizes,
  ptrViewIndices,
  ptrViewIDs,
  ptrData,
  ptrUserMsg
  )
  SMC_SIZET               winNum;
  SMC_SIZET*              numViews;
  SMC_DATAITEM_STRUCT**   ptrDIs;
  SMC_SIZET**             ptrViewSizes;
  SMC_SIZET**             ptrViewIndices;
  SMC_VIEW_ID**           ptrViewIDs;
  SMC_VALUE_UNIONP*       ptrData;
  SMC_CHAR**              ptrUserMsg;
#endif /* _SMC_PROTOTYPES */
{
  SMC_SIZET   i;
  SMC_SIZET   numDIs;
  
  switch (winNum) 
  { 
    case 1:
      *numViews = 3;
      numDIs = 21;
      break;
    case 2:  /* Device Window */
      *numViews = 2;
      numDIs = 12;
      break;
  }

  printf("winNum = %d\n", winNum);
  printf("numViews = %d\n", *numViews);
  printf("numDIs = %d\n", numDIs);
  
  (*ptrDIs) = (SMC_DATAITEM_STRUCT *)
              malloc(sizeof(SMC_DATAITEM_STRUCT) * numDIs);
  if ((*ptrDIs) == NULL)
  { 
    printf("ERR: failed to create DataItem array\n");
    return SMC_RET_FAILURE;
  }


  (*ptrViewSizes) = (SMC_SIZET *)malloc(sizeof(SMC_SIZET) * *numViews);
  if ((*ptrViewSizes) == NULL)
  { 
    printf("ERR: failed to create ViewSizes array\n");
    return SMC_RET_FAILURE;
  }


  (*ptrViewIndices) = (SMC_SIZET *)malloc(sizeof(SMC_SIZET) * *numViews);
  if ((*ptrViewIndices) == NULL)
  { 
    printf("ERR: failed to create ViewIndices array\n");
    return SMC_RET_FAILURE;
  }


  (*ptrViewIDs) = (SMC_VIEW_ID *)malloc(sizeof(SMC_VIEW_ID) * *numViews);
  if ((*ptrViewIDs) == NULL)
  { 
    printf("ERR: failed to create ViewIDs array\n");
    return SMC_RET_FAILURE;
  }


  for(i = 0; i < *numViews; i++)
    (*ptrViewIDs)[ i ] = 0;


  switch (winNum) 
  { 
     case 1:   /* Cache Window */
      {
        SMC_SIZET v01size = 2;
        static SMC_DATAITEM_STRUCT   v01di[] = {
        { SMC_NAME_SQL_SERVER_NAME,           SMC_STAT_VALUE_SAMPLE },
        { SMC_NAME_TIMESTAMP,                 SMC_STAT_VALUE_SAMPLE }
        };

        SMC_SIZET v02size = 8;
        static SMC_DATAITEM_STRUCT   v02di[] = { 
        { SMC_NAME_STP_HIT_PCT,               SMC_STAT_VALUE_SAMPLE },
        { SMC_NAME_STP_HIT_PCT,               SMC_STAT_VALUE_SESSION },
        { SMC_NAME_STP_LOGICAL_READ,          SMC_STAT_VALUE_SESSION },
        { SMC_NAME_STP_LOGICAL_READ,          SMC_STAT_RATE_SAMPLE },
        { SMC_NAME_STP_LOGICAL_READ,          SMC_STAT_RATE_SESSION },
        { SMC_NAME_STP_PHYSICAL_READ,         SMC_STAT_VALUE_SESSION },
        { SMC_NAME_STP_PHYSICAL_READ,         SMC_STAT_RATE_SAMPLE },
        { SMC_NAME_STP_PHYSICAL_READ,         SMC_STAT_RATE_SESSION }
        };
        
        SMC_SIZET v03size = 11;
        static SMC_DATAITEM_STRUCT   v03di[] = { 
        { SMC_NAME_PAGE_HIT_PCT,              SMC_STAT_VALUE_SAMPLE },
        { SMC_NAME_PAGE_HIT_PCT,              SMC_STAT_VALUE_SESSION },
        { SMC_NAME_PAGE_LOGICAL_READ,         SMC_STAT_VALUE_SESSION },
        { SMC_NAME_PAGE_LOGICAL_READ,         SMC_STAT_RATE_SAMPLE },
        { SMC_NAME_PAGE_LOGICAL_READ,         SMC_STAT_RATE_SESSION },
        { SMC_NAME_PAGE_PHYSICAL_READ,        SMC_STAT_VALUE_SESSION },
        { SMC_NAME_PAGE_PHYSICAL_READ,        SMC_STAT_RATE_SAMPLE },
        { SMC_NAME_PAGE_PHYSICAL_READ,        SMC_STAT_RATE_SESSION },
        { SMC_NAME_PAGE_WRITE,                SMC_STAT_VALUE_SESSION },
        { SMC_NAME_PAGE_WRITE,                SMC_STAT_RATE_SAMPLE },
        { SMC_NAME_PAGE_WRITE,                SMC_STAT_RATE_SESSION }
        };

        (*ptrViewSizes)[0] = v01size;
        (*ptrViewSizes)[1] = v02size;
        (*ptrViewSizes)[2] = v03size;
        
        (*ptrViewIndices)[0] = 0;
        (*ptrViewIndices)[1] = v01size;
        (*ptrViewIndices)[2] = v01size + v02size;
 
        for(i = 0; i< v01size; i++)
        {
          (*ptrDIs)[ ((*ptrViewIndices)[0] + i) ] = v01di[ i ];
        }
        for(i = 0; i< v02size; i++)
        {
          (*ptrDIs)[ ((*ptrViewIndices)[1] + i) ] = v02di[ i ];
        }
        for(i = 0; i< v03size; i++)
        {
          (*ptrDIs)[ ((*ptrViewIndices)[2] + i) ] = v03di[ i ];
        }
      }
      break;

    case 2:   /* Device Window */
      {
        SMC_SIZET v01size = 2;
        SMC_DATAITEM_STRUCT   v01di[] = {
        { SMC_NAME_SQL_SERVER_NAME,           SMC_STAT_VALUE_SAMPLE },
        { SMC_NAME_TIMESTAMP,                           SMC_STAT_VALUE_SAMPLE }
        };

        SMC_SIZET v02size = 10;
        SMC_DATAITEM_STRUCT   v02di[] = {
        { SMC_NAME_DEV_NAME,               SMC_STAT_VALUE_SAMPLE },
        { SMC_NAME_DEV_READ,                  SMC_STAT_VALUE_SESSION },
        { SMC_NAME_DEV_WRITE,                 SMC_STAT_VALUE_SESSION },
        { SMC_NAME_DEV_IO,                    SMC_STAT_VALUE_SESSION },
        { SMC_NAME_DEV_READ,                  SMC_STAT_RATE_SAMPLE },
        { SMC_NAME_DEV_WRITE,                 SMC_STAT_RATE_SAMPLE },
        { SMC_NAME_DEV_IO,                    SMC_STAT_RATE_SAMPLE },
        { SMC_NAME_DEV_READ,                  SMC_STAT_RATE_SESSION },
        { SMC_NAME_DEV_WRITE,                 SMC_STAT_RATE_SESSION },
        { SMC_NAME_DEV_IO,                    SMC_STAT_RATE_SESSION }
        };

        (*ptrViewSizes)[0] = v01size;
        (*ptrViewSizes)[1] = v02size;

        (*ptrViewIndices)[0] = 0;
        (*ptrViewIndices)[1] = v01size;



        for(i = 0; i< v01size; i++)
        {
          (*ptrDIs)[ ((*ptrViewIndices)[0] + i) ] = v01di[ i ];
        }
        for(i = 0; i< v02size; i++)
        {
          (*ptrDIs)[ ((*ptrViewIndices)[1] + i) ] = v02di[ i ];
        }
      }
      break;


}

  (*ptrData) = (SMC_VALUE_UNION *) malloc(MAX_BUFFER_SIZE);
  if ((*ptrData) == NULL)
  { 
    printf("ERR: failed to create Data array\n");
    return SMC_RET_FAILURE;
  }

  (*ptrUserMsg) = (SMC_CHAR *) malloc(MAX_BUFFER_SIZE);
  if ((*ptrUserMsg) == NULL)
  { 
    printf("ERR: failed to create UserMsg array\n");
    return SMC_RET_FAILURE;
  }

  return SMC_RET_SUCCESS;
}


/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: ShowViews()
*/
SMC_VOID
ShowViews(
#ifdef _SMC_PROTOTYPES
  SMC_SIZET               numViews,
  SMC_DATAITEM_STRUCT*    ptrDIs,
  SMC_SIZET*              ptrViewSizes,
  SMC_SIZET*              ptrViewIndices
  )
#else /* _SMC_PROTOTYPES */
  numViews,
  ptrDIs,
  ptrViewSizes,
  ptrViewIndices
  )
  SMC_SIZET               numViews;
  SMC_DATAITEM_STRUCT*    ptrDIs;
  SMC_SIZET*              ptrViewSizes;
  SMC_SIZET*              ptrViewIndices;
#endif /* _SMC_PROTOTYPES */
{
  SMC_SIZET           i;
  SMC_SIZET           j;
  for(i=0; i<numViews; i++)
  {
    printf("View Number: %d\n" , i+1 );
    for(j = 0; j<ptrViewSizes[ i ]; j++)
    { 
                  printf(" dataItemName=%d, dataItemStatType=%d\n", ptrDIs[ (ptrViewIndices[i] + j) ].dataItemName,
                  ptrDIs[ (ptrViewIndices[i] + j) ].dataItemStatType
                 );
    }  
    printf("\n");
  }

  return;
}


/* end testhist.c
////////////////////////////////////////////////////////////////////////////////
*/

