/*
//---------------------------------------------------------------------
//          Sybase Monitor
//          Confidential Property of Sybase, Inc.
//          (c) Copyright Sybase, Inc 1995,1996
//          All rights reserved
//-----------------------------------------------------------------
//
//  FILE: testmon.c
//
//  Sample program that will connect to a Monitor Server and request the
//  data associated with Stored Procedure executions.
//  This program filters for the 'sp_monitor' stored procedure and 
//  creates an alarm on the number of executions of that stored procedure
//  during each sample interval.
//
*/

/*
///////////////////////////////////////////////////////////////////////////////
//
//  Declarations
//
//
// standard includes
*/

/* stdio.h provides definition for sprintf() */
#include <stdio.h>
#include <string.h>

/* stdlib.h for sleep */
#ifndef _WIN
#include <stdlib.h>
#endif

#ifdef _WIN
#include <windows.h>
#endif

#include "example.h"

/* Header file for Monitor Client Library */

#include "mcpublic.h"

/* The following logic for determining whether function prototypes are supported
   is borrowed from the Sybase header file csconfig.h:
*/

#if defined(_MSC_VER)
#define CS_FORCE_PROTOTYPES	1
#endif

#if (__STDC__) || defined(CS_FORCE_PROTOTYPES)
# define _SMC_PROTOTYPES
#endif


#ifdef _SMC_PROTOTYPES
#define _SMC_PROTO(arglist) arglist
#else
#define _SMC_PROTO(ARGLIST) ()
#endif

/* function declarations */

#ifndef _WIN
unsigned sleep _SMC_PROTO((unsigned seconds));
#endif


/*
///////////////////////////////////////////////////////////////////////////////
//
//  Global Variables
*/
#define MAX_BUFFER_SIZE 512

SMC_SERVER_MODE		  serverMode = SMC_SERVER_M_LIVE;
SMC_SIZET        	  winNum = 13;		/* Window number. Non-configurable. */
SMC_SIZET         	  numRefreshes = 2;    	/* number of times data will be
						   sampled. Configurable. */
SMC_SIZET         	  refreshInterval = 5; 	/* number of seconds in between
						   samples. Configurable. */
SMC_BOOL          	  testFiltering = TRUE;
SMC_BOOL          	  testAlarms = TRUE;
SMC_CHAR	   	  MonserverName[31];
SMC_CHAR	    	  userName[31];
SMC_CHAR   	    	  userPassword[31];
SMC_CHAR   	    	  interfacesFile[257];
SMC_SIZET         	  version = 1001;


void
AlarmCallback _SMC_PROTO((
    SMC_CONNECT_ID              clientId,
    SMC_COMMAND_ID              commandId,
    SMC_VOIDP	                userData
  ));


SMC_VOID 
BufferData _SMC_PROTO((
  SMC_DATAITEM_NAME       name,
  SMC_DATAITEM_STATTYPE   statType,
  SMC_SIZET               colNum,
  SMC_VALUE_UNIONP        ptrData,
  SMC_SIZET               version,
  SMC_CHARP		  buffer
  ));

SMC_VOID
Cleanup _SMC_PROTO((
  SMC_CONNECT_ID          id,
  SMC_SIZET               numViews,
  SMC_DATAITEM_STRUCT*    ptrDIs,
  SMC_SIZET*              ptrViewSizes,
  SMC_SIZET*              ptrViewIndices,
  SMC_VIEW_ID*            ptrViewIDs,
  SMC_VALUE_UNIONP        ptrData,
  SMC_CHAR*               ptrUserMsg
  ));

SMC_RETURN_CODE
CreateFilters _SMC_PROTO((
  SMC_CONNECT_ID          client_id,
  SMC_VIEW_ID*		  ptrViewIDs,
  SMC_SIZET               winNum
  ));

SMC_RETURN_CODE
CreateAlarms _SMC_PROTO((
  SMC_CONNECT_ID          client_id,
  SMC_VIEW_ID*		  ptrViewIDs,
  SMC_SIZET               winNum
  ));

SMC_RETURN_CODE
CreateViews _SMC_PROTO((
  SMC_CONNECT_ID          id,
  SMC_SIZET               numViews,
  SMC_DATAITEM_STRUCT*    ptrDIs,
  SMC_SIZET*              ptrViewSizes,
  SMC_SIZET*              ptrViewIndices,
  SMC_VIEW_ID*            ptrViewIDs
  ));

SMC_RETURN_CODE
DoRefresh _SMC_PROTO((
  SMC_CONNECT_ID 	  id,
  SMC_CHAR*   		  ptrUserMsg,
  SMC_SIZET  		  refreshInterval
  ));

SMC_VOID
ErrorCallback _SMC_PROTO((
  SMC_CONNECT_ID	  id,
  SMC_COMMAND_ID	  commandId,
  SMC_VOIDP    		  userData
  ));

SMC_RETURN_CODE
ProcessRefresh _SMC_PROTO((
  SMC_CONNECT_ID          id,
  SMC_SIZET               numViews,
  SMC_DATAITEM_STRUCT*    ptrDIs,
  SMC_SIZET*              ptrViewSizes,
  SMC_SIZET*              ptrViewIndices,
  SMC_VIEW_ID*            ptrViewIDs,
  SMC_VALUE_UNIONP        ptrData,
  SMC_SIZET               version
  ));


SMC_RETURN_CODE
Setup _SMC_PROTO((
  SMC_SIZET               winNum,
  SMC_SIZET*              numViews,
  SMC_DATAITEM_STRUCT**   ptrDIs,
  SMC_SIZET**             ptrViewSizes,
  SMC_SIZET**             ptrViewIndices,
  SMC_VIEW_ID**           ptrViewIDs,
  SMC_VALUE_UNIONP*       ptrData,
  SMC_CHAR**              ptrUserMsg
  ));

SMC_VOID
ShowViews _SMC_PROTO((
  SMC_SIZET               numViews,
  SMC_DATAITEM_STRUCT*    ptrDIs,
  SMC_SIZET*              ptrViewSizes,
  SMC_SIZET*              ptrViewIndices
  ));

SMC_INT
main _SMC_PROTO((
  SMC_INT     argc,
  SMC_CHARP   argv[]
  ));


/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: main()
*/
SMC_INT
main(
#ifdef _SMC_PROTOTYPES
  SMC_INT     argc,
  SMC_CHARP   argv[]
  )
#else /* _SMC_PROTOTYPES */
  argc,
  argv
  )
  SMC_INT     argc;
  SMC_CHARP   argv[];
#endif /* _SMC_PROTOTYPES */

{
  SMC_INT i;

  SMC_RETURN_CODE        ret;
  SMC_CONNECT_ID         id = 0;
  SMC_CONNECT_ID*        ptrID = &id;
  SMC_SESSION_ID         sessId = 0;
  SMC_SESSION_ID*        ptrSessID = &sessId;
  SMC_SIZET              numViews = 0;
  SMC_DATAITEM_STRUCT*   ptrDIs = NULL;
  SMC_SIZET*             ptrViewSizes = NULL;
  SMC_SIZET*             ptrViewIndices = NULL;
  SMC_VIEW_ID*           ptrViewIDs = NULL;
  SMC_VALUE_UNIONP       ptrData = NULL;
  SMC_CHAR*              ptrUserMsg = NULL;
  SMC_SIZET              refreshCount = 1;

  SMC_VALUE_UNION	 prop;

  printf("**************************************************************\n");
  printf("******  C-language Test Driver for Monitor Client API  *******\n");
  printf("**************************************************************\n");

  if (argc > 1)
  {
    for (i=1; i<argc; i++)
    {
      printf("Unexpected command line argument: %s\n", argv[i]);
    }
  }

  strcpy(MonserverName, EX_MON_SERVER);

  strcpy(userName, EX_USERNAME);

  strcpy(userPassword, EX_PASSWORD);

  strcpy(interfacesFile, EX_INTERFACES);



  printf("*********    Testing smc_connect_alloc()   **********\n");
  ret = smc_connect_alloc(ErrorCallback,ptrID);
  if (ret != SMC_RET_SUCCESS)
  {
    printf("ERR: %d returned by smc_connect_alloc()\n", ret);
    return ((int)SMC_RET_FAILURE);
  }
  printf("*****  Testing smc_connect_props() -> serverMode  *****\n");
  prop.intValue = serverMode; 
  ret = smc_connect_props(id, SMC_PROP_ACT_SET, SMC_PROP_SERVERMODE, &prop,
                          SMC_UNUSED, NULL);
  if (ret != SMC_RET_SUCCESS)
  {
    printf("ERR: %d returned by smc_connect_props(SERVERMODE)\n", ret);
    return ((int)SMC_RET_FAILURE);
  }

  printf("*****  Testing smc_connect_props() -> serverName  *****\n");
  prop.stringValue = MonserverName; 
  ret = smc_connect_props(id, SMC_PROP_ACT_SET, SMC_PROP_SERVERNAME, &prop,
                          SMC_NULLTERM, NULL);
  if (ret != SMC_RET_SUCCESS)
  {
    printf("ERR: %d returned by smc_connect_props(SERVERNAME)\n", ret);
    return ((int)SMC_RET_FAILURE);
  }

  printf("*****  Testing smc_connect_props() -> userName  *****\n");
  prop.stringValue = userName; 
  ret = smc_connect_props(id, SMC_PROP_ACT_SET, SMC_PROP_USERNAME, &prop,
                          SMC_NULLTERM, NULL);
  if (ret != SMC_RET_SUCCESS)
  {
    printf("ERR: %d returned by smc_connect_props(USERNAME)\n", ret);
    return ((int)SMC_RET_FAILURE);
  }

  printf("*****  Testing smc_connect_props() -> userPassword  *****\n");
  prop.stringValue = userPassword; 
  ret = smc_connect_props(id, SMC_PROP_ACT_SET, SMC_PROP_PASSWORD, &prop,
                          SMC_NULLTERM, NULL);
  if (ret != SMC_RET_SUCCESS)
  {
    printf("ERR: %d returned by smc_connect_props(PASSWORD)\n", ret);
    return ((int)SMC_RET_FAILURE);
  }

  printf("*****  Testing smc_connect_props() -> interfacesFile  *****\n");
  prop.stringValue = interfacesFile;
  ret = smc_connect_props(id, SMC_PROP_ACT_SET, SMC_PROP_IFILE, &prop,
                          SMC_NULLTERM, NULL);
  if (ret != SMC_RET_SUCCESS)
  {
    printf("ERR: %d returned by smc_connect_props(PASSWORD)\n", (int) ret );
    return SMC_RET_FAILURE;
  }
 
 
  printf("*****  Testing smc_connect_props() -> ErrorCallback  *****\n");
  prop.voidpValue = (SMC_VOIDP) ErrorCallback; 
  ret = smc_connect_props(id, SMC_PROP_ACT_SET, SMC_PROP_ERROR_CALLBACK, &prop,
                          SMC_UNUSED, NULL);
  if (ret != SMC_RET_SUCCESS)
  {
    printf("ERR: %d returned by smc_connect_props(ERRORCALLBACK)\n", ret);
    return ((int)SMC_RET_FAILURE);
  }


  printf("********     Testing smc_connect_ex()    ************\n");
  ret = smc_connect_ex(id);
  if (ret != SMC_RET_SUCCESS)
  {
    printf("ERR: %d returned by smc_connect_ex()\n", ret);
    return ((int)SMC_RET_FAILURE);
  }

  printf("Connection Id: %d\n", id);

  ret = Setup(
              winNum,
              &numViews,
              &ptrDIs,
              &ptrViewSizes,
              &ptrViewIndices,
              &ptrViewIDs,
              &ptrData,
              &ptrUserMsg
             ); 
  if ( ret != SMC_RET_SUCCESS )
  {
    printf("ERR: %d returned by Setup()\n", ret);

    Cleanup(
       id,
       numViews,
       ptrDIs,
       ptrViewSizes,
       ptrViewIndices,
       ptrViewIDs,
       ptrData,
       ptrUserMsg);  

    return ((int)SMC_RET_FAILURE);
  }
            

  ShowViews(
            numViews,
            ptrDIs,
            ptrViewSizes,
            ptrViewIndices
           );


  ret = CreateViews(
                    id,
                    numViews,
                    ptrDIs,
                    ptrViewSizes,
                    ptrViewIndices,
                    ptrViewIDs
                   );
  if ( ret != SMC_RET_SUCCESS )
  {
    printf("ERR: %d returned by CreateViews()\n", ret);
    Cleanup(id, numViews, ptrDIs, ptrViewSizes, ptrViewIndices, ptrViewIDs, ptrData, ptrUserMsg);  
    return ((int)SMC_RET_FAILURE);
  }

  if (testFiltering)
  {
     CreateFilters(id, ptrViewIDs, winNum);
  }

  if (testAlarms)
  {
     CreateAlarms(id, ptrViewIDs, winNum);
  }

    while (refreshCount <= numRefreshes)
    {
      printf("doing refresh %d of %d\n", refreshCount, numRefreshes);
      sprintf(ptrUserMsg,"finished refresh %d of %d", refreshCount, numRefreshes);

      ret = DoRefresh(id, ptrUserMsg, refreshInterval);
      if ( ret != SMC_RET_SUCCESS )
      {
        printf("ERR: %d returned by DoRefresh()\n", ret);
        Cleanup(id, numViews, ptrDIs, ptrViewSizes, ptrViewIndices, ptrViewIDs, ptrData, ptrUserMsg);  
        return ((int)SMC_RET_FAILURE);
      }

      ret = ProcessRefresh(
                           id,
                           numViews,
                           ptrDIs,
                           ptrViewSizes,
                           ptrViewIndices,
                           ptrViewIDs,
                           ptrData,
                           version
                          );
      if ( ret != SMC_RET_SUCCESS )
      {
        printf("ERR: %d returned by ProcessRefresh()\n", ret);
        Cleanup(id, numViews, ptrDIs, ptrViewSizes, ptrViewIndices, ptrViewIDs, ptrData, ptrUserMsg);  
        return ((int)SMC_RET_FAILURE);
      }
    
      refreshCount++;

    }

   
  /* drop views and destroy local arrays */
  Cleanup(id, numViews, ptrDIs, ptrViewSizes, ptrViewIndices, ptrViewIDs, ptrData, ptrUserMsg);  
    printf( "*****   Testing   smc_close()          *****\n" );
    ret = smc_close(id, SMC_CLOSE_REQUEST);
    if ( ret != SMC_RET_SUCCESS )
    {
      printf("ERR: %d returned by smc_close()\n", (int)ret );
      return ((int)SMC_RET_FAILURE);
    }
 
    printf( "*****   Testing   smc_connect_drop()   *****\n" );
    ret = smc_connect_drop(id);
    if ( ret != SMC_RET_SUCCESS )
    {
      printf("ERR: %d returned by smc_connect_drop()\n", (int)ret );
      return ((int)SMC_RET_FAILURE);
    }


  printf("**********************   End of Test   ***********************\n");

  return ((int)SMC_RET_SUCCESS);
}


/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: BufferData()
*/
SMC_VOID
BufferData(
#ifdef _SMC_PROTOTYPES
  SMC_DATAITEM_NAME       name,
  SMC_DATAITEM_STATTYPE   statType,
  SMC_SIZET               colNum,
  SMC_VALUE_UNIONP        ptrData,
  SMC_SIZET               version,
  SMC_CHARP               rwBuffer
)
#else /* _SMC_PROTOTYPES */
  name,
  statType,
  colNum,
  ptrData,
  version,
  rwBuffer
)
  SMC_DATAITEM_NAME       name;
  SMC_DATAITEM_STATTYPE   statType;
  SMC_SIZET               colNum;
  SMC_VALUE_UNIONP        ptrData;
  SMC_SIZET               version;
  SMC_CHARP               rwBuffer;
#endif /* _SMC_PROTOTYPES */
{
  SMC_DATAITEM_TYPE   dataType;
  SMC_CHAR            Buffer[ MAX_BUFFER_SIZE ];
  SMC_INT             iVal;
  SMC_CHAR            lineText[ MAX_BUFFER_SIZE ];

  SMC_DATAITEM_STRUCT diStruct;
  diStruct.dataItemName = name;
  diStruct.dataItemStatType = statType;
  smc_get_dataitem_type(&diStruct,&dataType);

  switch(dataType)
  {
    case SMC_DI_TYPE_CHARP:

      if (name != SMC_NAME_STP_LINE_TEXT)
        sprintf(Buffer,"Col %d: \"%s\"\t",colNum,ptrData->stringValue);
      else
      {
        /* truncate STP_LINE_TEXT to first 15 chars (for display purposes) */
        strcpy(lineText, ptrData->stringValue);
        lineText[15] = '\0';
        strcat(lineText, "...");
        sprintf(Buffer,"Col %d: \"%s\"\t", colNum, lineText);
      }
      free(ptrData->stringValue);
      break;

    case SMC_DI_TYPE_DOUBLE:
      sprintf(Buffer,"Col %d: %f\t",colNum,ptrData->doubleValue);
      break;

    case SMC_DI_TYPE_ENUMS:
      iVal = ptrData->intValue;
      switch (name)
      {
        case SMC_NAME_LOCK_RESULT_SUMMARY:

          sprintf(Buffer,"Col %d: %d\t",colNum,iVal);
          break;

        case SMC_NAME_LOCK_RESULT:

          sprintf(Buffer,"Col %d: %d\t",colNum,iVal);
          break;

        case SMC_NAME_OBJ_TYPE:

          sprintf(Buffer,"Col %d: %d\t",colNum,iVal);
          break;

        case SMC_NAME_LOCK_STATUS:

          sprintf(Buffer,"Col %d: %d\t",colNum,iVal);
          break;

        case SMC_NAME_LOCK_TYPE:

          sprintf(Buffer,"Col %d: %d\t",colNum,iVal);
          break;

        case SMC_NAME_CUR_PROC_STATE:
        case SMC_NAME_PROC_STATE:
          
          if (version == 492)
            sprintf(Buffer,"Col %d: %d\t",colNum,iVal);
          else
            sprintf(Buffer,"Col %d: %d\t",colNum,iVal);
          break;

        default:

          sprintf(Buffer,"Col %d: \"ERR with name\t",colNum);
      }
      break;

    case SMC_DI_TYPE_LONG:
      sprintf(Buffer,"Col %d: %d\t",colNum,ptrData->longValue);
      break;

    case SMC_DI_TYPE_DATIM:
    case SMC_DI_TYPE_NONE:
    default:
      sprintf(Buffer,"Col %d: ERROR",colNum);
   }

   strcat(rwBuffer, Buffer);

   return;
}


/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: Cleanup() 
*/
SMC_VOID
Cleanup(
#ifdef _SMC_PROTOTYPES
  SMC_CONNECT_ID          id,
  SMC_SIZET               numViews,
  SMC_DATAITEM_STRUCT*    ptrDIs,
  SMC_SIZET*              ptrViewSizes,
  SMC_SIZET*              ptrViewIndices,
  SMC_VIEW_ID*            ptrViewIDs,
  SMC_VALUE_UNIONP        ptrData,
  SMC_CHAR*               ptrUserMsg
  )
#else /* _SMC_PROTOTYPES */
  id,
  numViews,
  ptrDIs,
  ptrViewSizes,
  ptrViewIndices,
  ptrViewIDs,
  ptrData,
  ptrUserMsg
  )
  SMC_CONNECT_ID          id;
  SMC_SIZET               numViews;
  SMC_DATAITEM_STRUCT*    ptrDIs;
  SMC_SIZET*              ptrViewSizes;
  SMC_SIZET*              ptrViewIndices;
  SMC_VIEW_ID*            ptrViewIDs;
  SMC_VALUE_UNIONP        ptrData;
  SMC_CHAR*               ptrUserMsg;
#endif /* _SMC_PROTOTYPES */
{
  SMC_SIZET   i;
  SMC_RETURN_CODE   ret;

  if ((ptrViewIDs != NULL) &&  (serverMode == SMC_SERVER_M_LIVE) )
  {
    for(i=0; i<numViews; i++)
    {  
      if ( ptrViewIDs[ i ] != 0)
      {
        ret = smc_drop_view(
                            id,
                            ptrViewIDs[ i ]
                           );
        if ( ret != SMC_RET_SUCCESS )
        {
           printf("ERR: %d returned by smc_drop_view()\n", ret);
        } 
      } 
    }
    free((char *)ptrViewIDs);
  }

  if (ptrDIs != NULL)
  {
    free((char *)ptrDIs);
  }

  if (ptrViewSizes != NULL)
  {
    free((char *)ptrViewSizes);
  }

  if (ptrViewIndices != NULL)
  {
    free((char *)ptrViewIndices);
  }

  if (ptrData != NULL)
  {
    free((char *)ptrData);
  }

  if (ptrUserMsg != NULL)
  {
    free((char *)ptrUserMsg);
  }

  return;
}


/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: CreateAlarms()
*/
SMC_RETURN_CODE
CreateAlarms( 
#ifdef _SMC_PROTOTYPES
  SMC_CONNECT_ID  client_id,
  SMC_VIEW_ID*    ptrViewIDs,
  SMC_SIZET       winNum
  )
#else /* _SMC_PROTOTYPES */
  client_id,
  ptrViewIDs,
  winNum
  )
  SMC_CONNECT_ID   client_id;
  SMC_VIEW_ID*     ptrViewIDs;
  SMC_SIZET        winNum;
#endif /* _SMC_PROTOTYPES */
{
  SMC_RETURN_CODE	 	 ret;
  SMC_DATAITEM_STRUCT    	 di;
  SMC_ALARM_ACTION_TYPE		 alarmAction;
  SMC_VALUE_UNION 		 alarmValue;
  SMC_DATAITEM_TYPE		 alarmDatatype;
  SMC_ALARM_ID                	 alarmId;
  SMC_ALARM_IDP               	 ptrAlarmId = &alarmId;
  SMC_VIEW_ID                    viewId;


  if ( winNum != 13 )
  {
    printf(" Alarms not supported for Window: %d\n", winNum);
    return SMC_RET_FAILURE;
  }

  switch (winNum)
  {
    case 13:
      printf(" Setting alarms for Window 13 on DataItem: SMC_NAME_STP_NUM_TIMES_EXECUTED\n");
      di.dataItemName =  SMC_NAME_STP_NUM_TIMES_EXECUTED;
      di.dataItemStatType = SMC_STAT_VALUE_SAMPLE;
      alarmAction = SMC_ALARM_A_NOTIFY;
      alarmDatatype = SMC_DI_TYPE_LONG;
      alarmValue.longValue = 3; 	/* Setting Alarm to be at 3 executions. */
      viewId = ptrViewIDs[1];
      break;
  }


  printf("************   Testing smc_create_alarm_ex()   *************\n");
/* Since we are attaching to a Monitor Server, only the Alarm Callback
   function will be called. It is up to the user to decide what action
   to take when this happens: write information to a log, execute a
   file,...
*/

  ret = smc_create_alarm_ex(
                          client_id,          /* connectionID  	        */
                          viewId,             /* viewid	    	    	*/
                          &di,                /* distruct   	    	*/
                          &alarmValue,        /* alarm value  	    	*/
			  alarmDatatype,      /* alarm datatype	    	*/
			  alarmAction,        /* alarm action 	    	*/
                          "an alarm was triggered", /* alarm action data */
			  NULL,		      /* user-supplied data     */
			  AlarmCallback,      /* callback function  	*/
                          ptrAlarmId          /* alarmIdHandle	    	*/
                         );
  if ( ret != SMC_RET_SUCCESS )
  {
    printf("ERR: %d returned by smc_create_alarm_ex()\n", ret);
    return SMC_RET_FAILURE;
  }

  return SMC_RET_SUCCESS;
}
  

/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: CreateFilters()
*/
SMC_RETURN_CODE
CreateFilters( 
#ifdef _SMC_PROTOTYPES
  SMC_CONNECT_ID   client_id,
  SMC_VIEW_ID*     ptrViewIDs,
  SMC_SIZET        winNum
  )
#else /* _SMC_PROTOTYPES */
  client_id,
  ptrViewIDs,
  winNum
  )
  SMC_CONNECT_ID   client_id;
  SMC_VIEW_ID*     ptrViewIDs;
  SMC_SIZET        winNum;
#endif /* _SMC_PROTOTYPES */
{
  SMC_RETURN_CODE          ret;
  SMC_DATAITEM_STRUCT      di;
  SMC_VALUE_UNION   	   filterVal;
  SMC_VALUE_UNIONP  	   ptrFilterVal = &filterVal;
  SMC_FILTER_ID            filterId;
  SMC_FILTER_IDP           ptrFilterId = &filterId;
  SMC_VIEW_ID              viewId;

  if ( winNum != 13 )
  {
    printf(" Filtering not supported for Window: %d\n", winNum);
    return SMC_RET_FAILURE;
  }


  switch (winNum)
  {
    case 13:
      printf(" Filtering Window 13 on DataItem: SMC_NAME_ACT_STP_NAME \n");
      di.dataItemName =  SMC_NAME_ACT_STP_NAME;
      di.dataItemStatType = SMC_STAT_VALUE_SAMPLE;
      viewId = ptrViewIDs[1];
      break;
  }


/* Filtering for the stored procedure 'sp_monitor' in any database by any owner */
  filterVal.stringValue = (char*)("%.%.sp_monitor");

  printf("************   Testing smc_create_filter()   *************\n");

  ret = smc_create_filter(
                          client_id,          /* connectionID  	        */
                          viewId,             /* viewid	    	    	*/
                          &di,                /* distruct   	    	*/
                          SMC_FILT_T_EQ,      /* filterType 	    	*/
                          ptrFilterVal,       /* filterValueListHandle	*/
                          1,                  /* filterValueListLength	*/
                          SMC_DI_TYPE_CHARP,  /* filterDataType     	*/
                          ptrFilterId         /* filterIdHandle	    	*/
                         );
  if ( ret != SMC_RET_SUCCESS )
  {
    printf("ERR: %d returned by smc_create_filter()\n", ret);
    return SMC_RET_FAILURE;
  }

  return SMC_RET_SUCCESS;
}
  

/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: CreateViews() 
*/
SMC_RETURN_CODE
CreateViews( 
#ifdef _SMC_PROTOTYPES
  SMC_CONNECT_ID          id,
  SMC_SIZET               numViews,
  SMC_DATAITEM_STRUCT*    ptrDIs,
  SMC_SIZET*              ptrViewSizes,
  SMC_SIZET*              ptrViewIndices,
  SMC_VIEW_ID*            ptrViewIDs
  )
#else /* _SMC_PROTOTYPES */
  id,
  numViews,
  ptrDIs,
  ptrViewSizes,
  ptrViewIndices,
  ptrViewIDs
  )
  SMC_CONNECT_ID          id;
  SMC_SIZET               numViews;
  SMC_DATAITEM_STRUCT*    ptrDIs;
  SMC_SIZET*              ptrViewSizes;
  SMC_SIZET*              ptrViewIndices;
  SMC_VIEW_ID*            ptrViewIDs;
#endif /* _SMC_PROTOTYPES */
{
  SMC_RETURN_CODE   	  ret;
  SMC_SIZET	    	  i;
  SMC_CHAR		  viewName[11];
  
  for(i=0; i<numViews; i++)
  {
      sprintf(viewName, "view%d", i);
      ret = smc_create_view(
                            id,
                            &(ptrDIs[ (ptrViewIndices[i]) ]),
                            ptrViewSizes[i],
			    viewName,
                            &(ptrViewIDs[i])
                           );
      if ( ret != SMC_RET_SUCCESS )
          {
           printf("ERR: %d returned by smc_create_view\n", ret);
           return SMC_RET_FAILURE;
          }
      printf("=== View Id: %d ===\n", ptrViewIDs[i]);
  }

  return SMC_RET_SUCCESS;
}


/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: DoRefresh()
*/
SMC_RETURN_CODE
DoRefresh(
#ifdef _SMC_PROTOTYPES
  SMC_CONNECT_ID  id,
  SMC_CHAR*       ptrUserMsg,
  SMC_SIZET       refreshInterval
  )
#else /* _SMC_PROTOTYPES */
  id,
  ptrUserMsg,
  refreshInterval
  )
  SMC_CONNECT_ID   id;
  SMC_CHAR*        ptrUserMsg;
  SMC_SIZET        refreshInterval;
#endif /* _SMC_PROTOTYPES */
{
  SMC_RETURN_CODE   ret;
#ifndef _WIN
  unsigned retcode;
#endif
  

#ifndef _WIN
  /*
  // sleep between issuing refreshes
  */
  printf(" sleeping for %d\n", refreshInterval);
  retcode = sleep( (unsigned)refreshInterval );
  printf(" retcode = %d\n", retcode);
#endif

    printf("************   Testing smc_refresh_ex()   *************\n");
    ret = smc_refresh_ex(id, 0);
    if ( ret != SMC_RET_SUCCESS )
    {
      printf("ERR: %d returned by smc_refresh_ex()\n", (int)ret );
      return SMC_RET_FAILURE;
    }
 
  return SMC_RET_SUCCESS;
}


/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: ErrorCallback()
*/
SMC_VOID
ErrorCallback(
#ifdef _SMC_PROTOTYPES
  SMC_CONNECT_ID id,
  SMC_COMMAND_ID commandId,
  SMC_VOIDP      userData
  )
#else /* _SMC_PROTOTYPES */
  id,
  commandId,
  userData
)
  SMC_CONNECT_ID id;
  SMC_COMMAND_ID commandId;
  SMC_VOIDP      userData;
#endif /* _SMC_PROTOTYPES */
{
  SMC_VALUE_UNION info;

  printf("Inside ErrorCallback()\n");

  printf("id = %d\n", id);
  printf("commandId = %d\n", commandId);
  if (userData != NULL)
    printf("userDataHandle = %s\n", userData);

  if (smc_get_command_info(id, commandId, SMC_INFO_ERR_MSG, &info,
                NULL) == SMC_RET_SUCCESS)
    printf("errMsg = %s\n", info.stringValue );
 
  if (smc_get_command_info(id, commandId, SMC_INFO_ERR_NUM, &info,
                NULL) == SMC_RET_SUCCESS)
    printf("errNum = %d\n", info.sizetValue );
 
  if (smc_get_command_info(id, commandId, SMC_INFO_ERR_SEVERITY, &info,
                NULL) == SMC_RET_SUCCESS)
    printf("errSeverity = %d\n", info.sizetValue );
 
  if (smc_get_command_info(id, commandId, SMC_INFO_ERR_MAPSEVERITY, &info,
                NULL) == SMC_RET_SUCCESS)
    printf("mapSeverity = %d\n", info.sizetValue );
 
  if (smc_get_command_info(id, commandId, SMC_INFO_ERR_SOURCE, &info,
                NULL) == SMC_RET_SUCCESS)
    printf("errSource = %d\n", info.sizetValue );
 
  if (smc_get_command_info(id, commandId, SMC_INFO_ERR_STATE, &info,
                NULL) == SMC_RET_SUCCESS)
    printf("errState = %d\n", info.sizetValue );

  return;  
}


/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: ProcessRefresh()
*/
SMC_RETURN_CODE
ProcessRefresh(
#ifdef _SMC_PROTOTYPES
  SMC_CONNECT_ID          id,
  SMC_SIZET               numViews,
  SMC_DATAITEM_STRUCT*    ptrDIs,
  SMC_SIZET*              ptrViewSizes,
  SMC_SIZET*              ptrViewIndices,
  SMC_VIEW_ID*            ptrViewIDs,
  SMC_VALUE_UNIONP        ptrData,
  SMC_SIZET               version
  )
#else /* _SMC_PROTOTYPES */
  id,
  numViews,
  ptrDIs,
  ptrViewSizes,
  ptrViewIndices,
  ptrViewIDs,
  ptrData,
  version
  )
  SMC_CONNECT_ID          id;
  SMC_SIZET               numViews;
  SMC_DATAITEM_STRUCT*    ptrDIs;
  SMC_SIZET*              ptrViewSizes;
  SMC_SIZET*              ptrViewIndices;
  SMC_VIEW_ID*            ptrViewIDs;
  SMC_VALUE_UNIONP        ptrData;
  SMC_SIZET               version;
#endif /* _SMC_PROTOTYPES */
{
  SMC_RETURN_CODE     ret;
  SMC_SIZET           i;
  SMC_SIZET           rowCount;
  SMC_SIZET*          ptrRowCount = &rowCount;
  SMC_SIZET           rowNum;
  SMC_SIZET           colNum;
  SMC_DATAITEM_NAME   diName;
  SMC_CHAR            rwBuffer[4000];
  SMC_CHAR            strBuffer[256];

  rwBuffer[0] = '\0';

  for(i=0; i<numViews;  i++)
  {
    printf("************   Testing smc_get_row_count()   *************\n");
    ret = smc_get_row_count(
                            id,
                            ptrViewIDs[ i ],
                            ptrRowCount  
                           );
    if ( ret != SMC_RET_SUCCESS )
    {
      printf("Error: %d returned by smc_get_row_count()", ret);
      return SMC_RET_FAILURE;
    }

    printf("Row Count: %d\n", rowCount);

    /*
    //  Print diNames in a View 
    */
    strcpy(rwBuffer, "\n");
    for(colNum = 0; colNum<ptrViewSizes[ i ]; colNum++)
    {
      diName = ptrDIs[ (ptrViewIndices[ i ] + colNum) ].dataItemName;
      sprintf(strBuffer, "Col %d : diName %d \t", colNum, diName);
      strcat(rwBuffer, strBuffer);
    }
    strcat(rwBuffer, "\n");
    printf("%s\n", rwBuffer);

    if (rowCount > 0)
    {
      printf("*************   Testing smc_get_dataitem_value()   *************\n");

      for (rowNum = 0; rowNum < rowCount; rowNum++)
      {
        rwBuffer[0] = '\0';
        printf("Row = %d\n", rowNum);
        for (colNum = 0; colNum < ptrViewSizes[ i ]; colNum++)
        {
          ret = smc_get_dataitem_value(
                                 id,
                                 ptrViewIDs[ i ],
                                 &(ptrDIs[ (ptrViewIndices[ i ] + colNum) ]), 
                                 rowNum,
                                 ptrData  
                                );
          if ( ret != SMC_RET_SUCCESS )
          {
            printf("ERR: %d returned by smc_get_dataitem_value()\n", ret);
            return SMC_RET_FAILURE;
          }
          else 
          {
            BufferData(
                      ptrDIs[ (ptrViewIndices[ i ] + colNum) ].dataItemName,
                      ptrDIs[ (ptrViewIndices[ i ] + colNum) ].dataItemStatType,
                      colNum,
                      ptrData,
                      version,
                      rwBuffer
                     ); 
          }
        }  /* end for(colNum... */
        printf("%s\n", rwBuffer);
      }  /* end for(rowNum... */
    }  /* end if(rowCount... */
  }

  return SMC_RET_SUCCESS;
}


/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: Setup() 
*/
SMC_RETURN_CODE
Setup(
#ifdef _SMC_PROTOTYPES
  SMC_SIZET               winNum,
  SMC_SIZET*              numViews,
  SMC_DATAITEM_STRUCT**   ptrDIs,
  SMC_SIZET**             ptrViewSizes,
  SMC_SIZET**             ptrViewIndices,
  SMC_VIEW_ID**           ptrViewIDs,
  SMC_VALUE_UNIONP*       ptrData,
  SMC_CHAR**              ptrUserMsg
  )
#else /* _SMC_PROTOTYPES */
  winNum,
  numViews,
  ptrDIs,
  ptrViewSizes,
  ptrViewIndices,
  ptrViewIDs,
  ptrData,
  ptrUserMsg
  )
  SMC_SIZET               winNum;
  SMC_SIZET*              numViews;
  SMC_DATAITEM_STRUCT**   ptrDIs;
  SMC_SIZET**             ptrViewSizes;
  SMC_SIZET**             ptrViewIndices;
  SMC_VIEW_ID**           ptrViewIDs;
  SMC_VALUE_UNIONP*       ptrData;
  SMC_CHAR**              ptrUserMsg;
#endif /* _SMC_PROTOTYPES */
{
  SMC_SIZET   i;
  SMC_SIZET   numDIs;
  
  switch (winNum) 
  { 
    case 1:    /* Cache Window */
      *numViews = 3;
      numDIs = 21;
      break;
    case 13:   /* Stored Procedure Window */
      *numViews = 2;
      numDIs = 10;
      break;
  }

  printf("winNum = %d\n", winNum);
  printf("numViews = %d\n", *numViews);
  printf("numDIs = %d\n", numDIs);
  
  (*ptrDIs) = (SMC_DATAITEM_STRUCT *)
	      malloc(sizeof(SMC_DATAITEM_STRUCT) * numDIs);
  if ((*ptrDIs) == NULL)
  { 
    printf("ERR: failed to create DataItem array\n");
    return SMC_RET_FAILURE;
  }


  (*ptrViewSizes) = (SMC_SIZET *)malloc(sizeof(SMC_SIZET) * *numViews);
  if ((*ptrViewSizes) == NULL)
  { 
    printf("ERR: failed to create ViewSizes array\n");
    return SMC_RET_FAILURE;
  }


  (*ptrViewIndices) = (SMC_SIZET *)malloc(sizeof(SMC_SIZET) * *numViews);
  if ((*ptrViewIndices) == NULL)
  { 
    printf("ERR: failed to create ViewIndices array\n");
    return SMC_RET_FAILURE;
  }


  (*ptrViewIDs) = (SMC_VIEW_ID *)malloc(sizeof(SMC_VIEW_ID) * *numViews);
  if ((*ptrViewIDs) == NULL)
  { 
    printf("ERR: failed to create ViewIDs array\n");
    return SMC_RET_FAILURE;
  }


  for(i = 0; i < *numViews; i++)
    (*ptrViewIDs)[ i ] = 0;


  switch (winNum) 
  { 
     case 1:   /* Cache Window */
      {
        SMC_SIZET v01size = 2;
        static SMC_DATAITEM_STRUCT   v01di[] = {
        { SMC_NAME_SQL_SERVER_NAME,           SMC_STAT_VALUE_SAMPLE },
        { SMC_NAME_TIMESTAMP,	         SMC_STAT_VALUE_SAMPLE }
        };

        SMC_SIZET v02size = 8;
        static SMC_DATAITEM_STRUCT   v02di[] = { 
        { SMC_NAME_STP_HIT_PCT,               SMC_STAT_VALUE_SAMPLE },
        { SMC_NAME_STP_HIT_PCT,               SMC_STAT_VALUE_SESSION },
        { SMC_NAME_STP_LOGICAL_READ,          SMC_STAT_VALUE_SESSION },
        { SMC_NAME_STP_LOGICAL_READ,          SMC_STAT_RATE_SAMPLE },
        { SMC_NAME_STP_LOGICAL_READ,          SMC_STAT_RATE_SESSION },
        { SMC_NAME_STP_PHYSICAL_READ,         SMC_STAT_VALUE_SESSION },
        { SMC_NAME_STP_PHYSICAL_READ,         SMC_STAT_RATE_SAMPLE },
        { SMC_NAME_STP_PHYSICAL_READ,         SMC_STAT_RATE_SESSION }
        };
        
        SMC_SIZET v03size = 11;
        static SMC_DATAITEM_STRUCT   v03di[] = { 
        { SMC_NAME_PAGE_HIT_PCT,              SMC_STAT_VALUE_SAMPLE },
        { SMC_NAME_PAGE_HIT_PCT,              SMC_STAT_VALUE_SESSION },
        { SMC_NAME_PAGE_LOGICAL_READ,         SMC_STAT_VALUE_SESSION },
        { SMC_NAME_PAGE_LOGICAL_READ,         SMC_STAT_RATE_SAMPLE },
        { SMC_NAME_PAGE_LOGICAL_READ,         SMC_STAT_RATE_SESSION },
        { SMC_NAME_PAGE_PHYSICAL_READ,        SMC_STAT_VALUE_SESSION },
        { SMC_NAME_PAGE_PHYSICAL_READ,        SMC_STAT_RATE_SAMPLE },
        { SMC_NAME_PAGE_PHYSICAL_READ,        SMC_STAT_RATE_SESSION },
        { SMC_NAME_PAGE_WRITE,                SMC_STAT_VALUE_SESSION },
        { SMC_NAME_PAGE_WRITE,                SMC_STAT_RATE_SAMPLE },
        { SMC_NAME_PAGE_WRITE,                SMC_STAT_RATE_SESSION }
        };

        (*ptrViewSizes)[0] = v01size;
        (*ptrViewSizes)[1] = v02size;
        (*ptrViewSizes)[2] = v03size;
        
        (*ptrViewIndices)[0] = 0;
        (*ptrViewIndices)[1] = v01size;
        (*ptrViewIndices)[2] = v01size + v02size;
 
        for(i = 0; i< v01size; i++)
        {
          (*ptrDIs)[ ((*ptrViewIndices)[0] + i) ] = v01di[ i ];
        }
        for(i = 0; i< v02size; i++)
        {
          (*ptrDIs)[ ((*ptrViewIndices)[1] + i) ] = v02di[ i ];
        }
        for(i = 0; i< v03size; i++)
        {
          (*ptrDIs)[ ((*ptrViewIndices)[2] + i) ] = v03di[ i ];
        }
      }
      break;

    case 13:   /* Stored Procedure Activity Window */
      {
        SMC_SIZET v01size = 3;
        SMC_DATAITEM_STRUCT   v01di[] = {
        { SMC_NAME_SQL_SERVER_NAME,           SMC_STAT_VALUE_SAMPLE },
        { SMC_NAME_TIMESTAMP,                 SMC_STAT_VALUE_SAMPLE },
        { SMC_NAME_ELAPSED_TIME,              SMC_STAT_VALUE_SESSION }
        };

        SMC_SIZET v02size = 7;
        SMC_DATAITEM_STRUCT   v02di[] = {
        { SMC_NAME_ACT_STP_DB_ID,              SMC_STAT_VALUE_SAMPLE },
        { SMC_NAME_ACT_STP_ID,                SMC_STAT_VALUE_SAMPLE },
        { SMC_NAME_STP_NUM_TIMES_EXECUTED,    SMC_STAT_VALUE_SAMPLE },
        { SMC_NAME_ACT_STP_DB_NAME,           SMC_STAT_VALUE_SAMPLE },
        { SMC_NAME_ACT_STP_NAME,              SMC_STAT_VALUE_SAMPLE },
        { SMC_NAME_STP_ELAPSED_TIME,          SMC_STAT_AVG_SAMPLE },
        { SMC_NAME_STP_ELAPSED_TIME,          SMC_STAT_AVG_SESSION }
        };

        (*ptrViewSizes)[0] = v01size;
        (*ptrViewSizes)[1] = v02size;

        (*ptrViewIndices)[0] = 0;
        (*ptrViewIndices)[1] = v01size;

        for(i = 0; i< v01size; i++)
        {
          (*ptrDIs)[ ((*ptrViewIndices)[0] + i) ] = v01di[ i ];
        }
        for(i = 0; i< v02size; i++)
        {
          (*ptrDIs)[ ((*ptrViewIndices)[1] + i) ] = v02di[ i ];
        }
      }
      break;

}

  (*ptrData) = (SMC_VALUE_UNION *) malloc(MAX_BUFFER_SIZE);
  if ((*ptrData) == NULL)
  { 
    printf("ERR: failed to create Data array\n");
    return SMC_RET_FAILURE;
  }

  (*ptrUserMsg) = (SMC_CHAR *) malloc(MAX_BUFFER_SIZE);
  if ((*ptrUserMsg) == NULL)
  { 
    printf("ERR: failed to create UserMsg array\n");
    return SMC_RET_FAILURE;
  }

  return SMC_RET_SUCCESS;
}


/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: ShowViews()
*/
SMC_VOID
ShowViews(
#ifdef _SMC_PROTOTYPES
  SMC_SIZET               numViews,
  SMC_DATAITEM_STRUCT*    ptrDIs,
  SMC_SIZET*              ptrViewSizes,
  SMC_SIZET*              ptrViewIndices
  )
#else /* _SMC_PROTOTYPES */
  numViews,
  ptrDIs,
  ptrViewSizes,
  ptrViewIndices
  )
  SMC_SIZET               numViews;
  SMC_DATAITEM_STRUCT*    ptrDIs;
  SMC_SIZET*              ptrViewSizes;
  SMC_SIZET*              ptrViewIndices;
#endif /* _SMC_PROTOTYPES */
{
  SMC_SIZET           i;
  SMC_SIZET           j;
  for(i=0; i<numViews; i++)
  {
    printf("View Number: %d\n" , i+1 );
    for(j = 0; j<ptrViewSizes[ i ]; j++)
    { 
                  printf(" dataItemName=%d, dataItemStatType=%d\n", ptrDIs[ (ptrViewIndices[i] + j) ].dataItemName,
                  ptrDIs[ (ptrViewIndices[i] + j) ].dataItemStatType
                 );
    }  
    printf("\n");
  }

  return;
}


/*
///////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: AlarmCallback()
*/
void
AlarmCallback(
#ifdef _SMC_PROTOTYPES
    SMC_CONNECT_ID              clientId,
    SMC_COMMAND_ID              commandId,
    SMC_VOIDP	                userData
)
#else /* _SMC_PROTOTYPES */
    clientId,
    commandId,
    userData
)
    SMC_CONNECT_ID              clientId;
    SMC_COMMAND_ID              commandId;
    SMC_VOIDP	                userData;
#endif /* _SMC_PROTOTYPES */
{
  SMC_VALUE_UNION info;
  SMC_RETURN_CODE ret;
 
  printf("**********   AlarmCallback() fired   **********\n");
 
  printf("connectId = %d\n", clientId );
  printf("commandId = %d\n", commandId );
  if (userData != NULL)
    printf("userData = %s\n", userData);
 
  if (smc_get_command_info(clientId, commandId, SMC_INFO_ALARM_ACTION_DATA,
                           &info, NULL) == SMC_RET_SUCCESS)
  {
    if (info.stringValue != NULL)
      printf("ptrAlarmActionData = %s\n", info.stringValue );
  }


  if (smc_get_command_info(clientId, commandId, SMC_INFO_ALARM_VIEWID, &info,
                        NULL) == SMC_RET_SUCCESS)
    printf("viewId = %d\n", info.sizetValue);
 
 
  if (smc_get_command_info(clientId, commandId, SMC_INFO_ALARM_ALARMID, &info,
                        NULL) == SMC_RET_SUCCESS)
    printf("alarmId = %d\n", info.sizetValue);
 
  if (smc_get_command_info(clientId, commandId, SMC_INFO_ALARM_VALUE_DATATYPE,
                           &info, NULL) == SMC_RET_SUCCESS)
  {
    SMC_INT dataType = info.intValue;
    if (smc_get_command_info(clientId, commandId, SMC_INFO_ALARM_CURRENT_VALUE,
                             &info, NULL) == SMC_RET_SUCCESS)
    {
      switch (dataType)
      {
        case SMC_DI_TYPE_INT:
          printf("currentValue = %d\n", info.intValue);
          break;
        case SMC_DI_TYPE_LONG:
          printf("currentValue = %d\n", info.longValue);
          break;
        case SMC_DI_TYPE_DOUBLE:
          printf("currentValue = %ld\n", info.doubleValue);
          break;
        default:
          break;
      }
    }

    if (smc_get_command_info(clientId,commandId,SMC_INFO_ALARM_THRESHOLD_VALUE, 
                             &info, NULL) == SMC_RET_SUCCESS)
    {
      switch (dataType)
      {
        case SMC_DI_TYPE_INT:
          printf("thresholdValue = %d\n", info.intValue);
          break;
        case SMC_DI_TYPE_LONG:
          printf("thresholdValue = %d\n", info.longValue);
          break;
        case SMC_DI_TYPE_DOUBLE:
          printf("thresholdValue = %ld\n", info.doubleValue);
          break;
        default:
          break;
      }
    }
  }
 
 
  if (smc_get_command_info(clientId, commandId, SMC_INFO_ALARM_DATAITEM, &info,
                        NULL) == SMC_RET_SUCCESS)
  {
    printf("DataItem name = %d\n",
            (int)((SMC_DATAITEM_STRUCTP) info.voidpValue)->dataItemName );
    printf("DataItem stat = %d\n",
            (int)((SMC_DATAITEM_STRUCTP) info.voidpValue)->dataItemStatType );
  }
 
 
  if (smc_get_command_info(clientId, commandId, SMC_INFO_ALARM_ROW, &info,
                        NULL) == SMC_RET_SUCCESS)
    printf("Row Number = %d\n", info.sizetValue);
 
 
  if (smc_get_command_info(clientId,commandId, SMC_INFO_ALARM_TIMESTAMP, &info,
                        NULL) == SMC_RET_SUCCESS)
    printf("TimeStamp = %s\n", info.stringValue);

  printf("******************************************************************\n");
}


/* end testmon.c
////////////////////////////////////////////////////////////////////////////////
*/
