/*
**  DO NOT MODIFY BELOW THIS COMMENT UNTIL SAMPLE CODE
*/
/*
** This DLL is used only with the Security Control Layer.
**
** The namespace DLL may reside anywhere in the LD_LIBRARY_PATH or SHLIB_PATH 
** variable for unix machines,  and in the PATH variable for NT.
** However, for Server the DLL may also reside in $SYBASE/lib, $SYBASE/symlib
** for unix, and $SYBASE/dll for NT.
*/
#define SYBNAMEMAP		"libsybmapname"

#ifndef DLL_SUFFIX

#if HP9000 /* HP unix machines */
#define DLL_SUFFIX	".sl		/* libsybmapname.sl */
#elif (NT || NT_ALPHA) /* NT machines */
#define DLL_SUFFIX	".dll"		/* libsybmapname.dll */
#elif RS6000 /* IBM posix machines */
#define DLL_SUFFIX	".so.a" 	/* libsybmapname.so.a */
#else /* all other unix machines */
#define DLL_SUFFIX	".so"		/* libsybmapname.so */
#endif

#endif /* DLL_SUFFIX */ 

#ifndef MAXCHAR
#define MAXCHAR	255
#endif /* MAXCHAR */

#ifndef MAXNAME
#define MAXNAME 30
#endif /* MAXNAME */

#ifndef DLL_HANDLE
#if NT
#define DLL_HANDLE	HINSTANCE
#else
#define DLL_HANDLE	void *
#endif
#endif /* DLL_HANDLE */

#if (NT || NT_ALPHA)
#ifndef CDECL
#define CDECL __cdecl
#endif
#endif /* NT || NT_ALPHA */

/*
** #define's for which application types are reserved by Sybase, Inc.
*/
#define ASERVERE "Adaptive Server Enterprise"

/*
** app: application name (e.g. Adapative Server Enterprise )
**	See above for current sybase uses.
** name: instance of application (e.g. server name in interfaces file)
**	typically your interfaces file entry for the application
** mech: which underlying security mechanism the server was booted with
**	mechanism name found in interfaces file, and listed in libtcl.cfg
** errmsg: output of error message if there is one.
**	MUST return non NULL value for mapping function.
*/
typedef struct syb_namemap_struct {
	char	app[MAXCHAR];  /* application type must be terminated */
	char	name[MAXCHAR]; /* name of the appliation server */
	char	mech[MAXCHAR]; /* security mechanism name */
	char 	errmsg[MAXCHAR]; /* OUTPUT: error message from DLL if any */
} NAMEMAPTYPE;

/*
** IMPORTANT:
**
** 	Expecting function to look as follows.
**	int syb__map_name(NAMEMAPTYPE *, void *, int, void *, int *)
**
** Return Values:
**	TRUE	(> 0)	if name was mapped
**	FALSE	(= 0)	if name was not mapped
**	ERROR	(< 0)	if an error/error message was returned;
**			Server: the error message will be printed into errorlog
*/
#define NAMEMAPFUNC	"syb__map_name"

#define ERROR	-1
#define FALSE 	0
#define TRUE	1

/*
** Compiling methods:  for unix "cc -c sybmapname.c" to create the .o file
** Linking methods:    for unix "ld -o libsybmapname.so -G -dy sybmapname.o"
*/

/*
** DO NOT MODIFY ABOVE THIS COMMENT
*/

/*
** Start of sample DLL code
*/

#include	<stdio.h>
#include	<ctype.h>

/*
**  MAP_NAME
**
**	This routine does the mapping of characters in the principal
**	name.  It is assumed that the underlying security mechanisms 
**	do not do any conversions of character sets, and therefore neither 
**	will this DLL.  However, if in the future this is not the case, then 
**	the character set name and character width will need to be passed in 
**	to accomodate conversions.
**
**  Parameters:
**  app		-- Account name from security service (e.g. "Adapative Server")
**  name	-- Name of the application from the interfaces file.
**  mech	-- Underlying security mechanism name.
**  errmsg	-- [OUTPUT] error message buffer
**
**  NOTE: Avoid use of 'goto' statements since, depending on the platform,
**	  these can have varying side effects.
**
**	  SERVER: "orig" is not null terminated.
**	  SERVER: "mapped" should not be null terminated
**
**  Returns:
**	True if the mapping succeeds
**	False if there was a problem
*/
#if (NT || NT_ALPHA)
int
CDECL syb_map_name(NAMEMAPTYPE *protocol, void *orig, int origlen, void *mapped, 
		int *mappedlen)
#else
int
syb__map_name(NAMEMAPTYPE *protocol, void *orig, int origlen, void *mapped, 
		int *mappedlen)
#endif
{
	int 	retval;
	int	i;
	char	buf[MAXCHAR];
	char	mapbuf[MAXNAME+1];

	/* Initialize variables */
	retval = FALSE;
	strncpy(buf, (char *) orig, origlen);
	buf[origlen] = '\0';
		
	if (!strcmp(protocol->app, ASERVERE))
	{
		/*
		** Check "orig" for invalid characters or string you want
		** to change and pass up to the application.
		*/
		for (i = 0; i < origlen; i++)
		{
			/* Do character mapping here */
			if (buf[i] == '#')	/* character searching for */
				mapbuf[i] = '_';/* character mapped to */
			else
				mapbuf[i] = buf[i];
		}

		/* null terminate the mapped buf*/
		mapbuf[origlen] = '\0';

		if (strcmp(mapbuf, buf))
		{
			/* mappedlen must be 30 characters or less */
			*mappedlen = strlen(mapbuf);
			strncpy(mapped, mapbuf, *mappedlen);
			retval = TRUE;
		} else {
			/* sample error if no mapping was done at all */
			strcpy(protocol->errmsg, "Error message name map");
			retval = ERROR;
		}
	}

	return (retval);
}
