//
// Confidential property of Sybase, Inc.
// (c) Copyright Sybase, Inc. 1997.
// All rights reserved
//
package sample;
import java.sql.*;

/**
* <P>This interface is an extension of the JDBC 1.0 
* <CODE>java.sql.ResultSet</CODE> interface.  The purpose is to define
* the additional methods for scrollable behavior.  The methods are duplicates
* of the methods added to the JDBC 2.0 API for the <CODE>ResultSet</CODE>
* interface.  However, this interface is a subset of the JDBC 2.0 ResultSet.
* This interface does not provide update or delete methods.
* <P>
* The constants defined on this interface have the same semantics as those
* found in the JDBC 2.0 API for java.sql.ResultSet. No guarantee is made
* that ScrollableResultSet.FETCH_FORWARD == ResultSet.FETCHFORWARD.
*
* @see java.sql.ResultSet
*/
public interface ScrollableResultSet extends java.sql.ResultSet
{

    int FETCH_FORWARD = 10;
    int FETCH_REVERSE = 11;
    int FETCH_UNKNOWN = 12;

    int CONCUR_READ_ONLY = 20;
    int CONCUR_UPDATABLE = 21;

    int TYPE_FORWARD_ONLY = 30;
    int TYPE_SCROLL_INSENSITIVE = 31;

    // Cursor movement

    boolean previous() throws SQLException;

    boolean absolute(int row) throws SQLException;
    boolean relative(int rows) throws SQLException;

    boolean first() throws SQLException;    
    boolean last() throws SQLException;

    void beforeFirst() throws SQLException;
    void afterLast() throws SQLException;


    // Cursor location
    boolean isFirst() throws SQLException;
    boolean isLast() throws SQLException;

    boolean isBeforeFirst() throws SQLException;
    boolean isAfterLast() throws SQLException;


    // Cursor configuration
    int getFetchSize() throws SQLException;
    void setFetchSize(int rows) throws SQLException;
    int getFetchDirection() throws SQLException;
    void setFetchDirection(int direction) throws SQLException;

    int getType() throws SQLException;    
    int getConcurrency() throws SQLException;
    int getRow() throws SQLException;
}
